#include "cps.h"

/*==============================================
 Dimahoo / Great Mahou Daisakusen
 ==============================================*/

static struct BurnInputInfo DrvInputList[] =
{
  {"P1 Coin"       , 0, CpsInp020+4, "p1 coin"  },
  {"P1 Start"      , 0, CpsInp020+0, "p1 start" },
  {"P1 Up"         , 0, CpsInp001+3, "p1 up"    },
  {"P1 Down"       , 0, CpsInp001+2, "p1 down"  },
  {"P1 Left"       , 0, CpsInp001+1, "p1 left"  },
  {"P1 Right"      , 0, CpsInp001+0, "p1 right" },
  {"P1 Shot"       , 0, CpsInp001+4, "p1 fire 1"},
  {"P1 Bomb"       , 0, CpsInp001+5, "p1 fire 2"},
  {"P1 Shot (auto)", 0, CpsInp001+6, "p1 fire 3"},

  {"P2 Coin"       , 0, CpsInp020+5, "p2 coin"  },
  {"P2 Start"      , 0, CpsInp020+1, "p2 start" },
  {"P2 Up"         , 0, CpsInp000+3, "p2 up"    },
  {"P2 Down"       , 0, CpsInp000+2, "p2 down"  },
  {"P2 Left"       , 0, CpsInp000+1, "p2 left"  },
  {"P2 Right"      , 0, CpsInp000+0, "p2 right" },
  {"P2 Shot"       , 0, CpsInp000+4, "p2 fire 1"},
  {"P2 Bomb"       , 0, CpsInp000+5, "p2 fire 2"},
  {"P2 Shot (auto)", 0, CpsInp000+6, "p2 fire 3"},

  {"Reset"         , 0, &CpsReset  , "reset"    },
  {"Diagnostic"    , 0, CpsInp021+1, "diag"     },
  {"Service"       , 0, CpsInp021+2, "service"  },
};

STDINPUTINFO(Drv);

static int DrvInit()
{
  int nRet=0; int i=0; unsigned char *pqs=NULL;

  Cps=2;
  nCpsRomLen=  4*0x080000;
  nCpsCodeLen= 1*0x080000;
  nCpsGfxLen=  4*0x400000;
  nCpsZRomLen= 2*0x020000;
  nCpsQSamLen= 2*0x400000;
  nRet=CpsInit(); if (nRet!=0) return 1;

  for (i=0;i<4;i++)
  { nRet=BurnLoadRom(CpsRom+0x080000*i,1+i,1); if (nRet!=0) return 1; }

  memcpy(CpsCode,CpsRom,nCpsCodeLen);
  nRet=BurnXorRom(CpsCode,0,1); if (nRet!=0) return 1;

  nRet=Cps2LoadTiles(CpsGfx,5);

  nRet=BurnLoadRom(CpsZRom,9,1);
  nRet=BurnLoadRom(CpsZRom+0x20000,10,1);

  pqs=(unsigned char *)CpsQSam;
  nRet=BurnLoadRom(pqs         ,11,1);
  nRet=BurnLoadRom(pqs+0x400000,12,1);
  BurnByteswap(pqs,nCpsQSamLen);

  nRet=CpsRunInit(); if (nRet!=0) return 1;
  return 0;
}

static int DrvExit()
{
  CpsRunExit();
  CpsExit();
  nCpsQSamLen=0;
  nCpsZRomLen=0;
  nCpsGfxLen=0;
  nCpsCodeLen=0;
  nCpsRomLen=0;
  Cps=0;

  return 0;
}


/*==========
 Dimahoo (USA)
==========*/


static struct BurnRomInfo DimahooRomDesc[]=
{
  {"gmdux.03" ,0x80000,0x9F820809, 0x10},

  {"gmdu.03"  ,0x80000,0x43BCB15F, 0x10},
  {"gmd.04"   ,0x80000,0x37485567, 0x10},
  {"gmd.05"   ,0x80000,0xDA269FFB, 0x10},
  {"gmd.06"   ,0x80000,0x55B483C9, 0x10},

  {"gmd.13"  ,0x400000,0x80DD19F0,    1},
  {"gmd.15"  ,0x400000,0xDFD93A78,    1},
  {"gmd.17"  ,0x400000,0x16356520,    1},
  {"gmd.19"  ,0x400000,0xDFC33031,    1},

  {"gmd.01"  ,0x020000,0x3F9BC985,    2},
  {"gmd.02"  ,0x020000,0x3FD39DDE,    2},

  {"gmd.11"  ,0x400000,0x06A65542,    2},
  {"gmd.12"  ,0x400000,0x50BC7A31,    2},
};

STD_ROM_PICK(Dimahoo) STD_ROM_FN(Dimahoo)

struct BurnDriver BurnDrvCpsDimahoo=
{
  {"dimahoo" ,"Dimahoo (US 000121)",NULL,"Capcom","CPS2","2000",NULL, NULL},
  BDF_GAME_WORKING | BDF_ROTATE_GRAPHICS_CCW, 2, HARDWARE_CAPCOM_CPS2,
  NULL, DimahooRomInfo, DimahooRomName, DrvInputInfo, NULL,
  DrvInit,DrvExit,Cps2Frame,CpsRedraw,CpsAreaScan,
  &CpsRecalcPal,224,384,3,4
};

/*===============
 Great Mahou Daisakusen
===============*/


static struct BurnRomInfo gmdjRomDesc[]=
{
	{"gmdjx.03" ,0x80000,0xC0C8Fd1E, 0x10},

	{"gmdj.03"  ,0x80000,0xCD6979E3, 0x10},
	{"gmd.04"   ,0x80000,0x37485567, 0x10},
	{"gmd.05"   ,0x80000,0xDA269FFB, 0x10},
	{"gmd.06"   ,0x80000,0x55B483C9, 0x10},

	{"gmd.13"  ,0x400000,0x80DD19F0,    1},
	{"gmd.15"  ,0x400000,0xDFD93A78,    1},
	{"gmd.17"  ,0x400000,0x16356520,    1},
	{"gmd.19"  ,0x400000,0xDFC33031,    1},

	{"gmd.01"  ,0x020000,0x3F9BC985,    2},
	{"gmd.02"  ,0x020000,0x3FD39DDE,    2},

	{"gmd.11"  ,0x400000,0x06A65542,    2},
	{"gmd.12"  ,0x400000,0x50BC7A31,    2},
};

STD_ROM_PICK(gmdj) STD_ROM_FN(gmdj)

struct BurnDriver BurnDrvCpsGreatMahouDaiJ=
{
  {"gmahou" ,"Great Mahou Daisakusen (Japan 000121)",NULL,"Capcom","CPS2","2000","dimahoo", NULL},
  BDF_GAME_WORKING | BDF_ROTATE_GRAPHICS_CCW, 2, HARDWARE_CAPCOM_CPS2,
  NULL, gmdjRomInfo, gmdjRomName, DrvInputInfo, NULL,
  DrvInit,DrvExit,Cps2Frame,CpsRedraw,CpsAreaScan,
  &CpsRecalcPal,224,384,3,4
};

