// Cave hardware sprites
#include "cave.h"

unsigned char* CaveSpriteROM = NULL;
unsigned char* CaveSpriteRAM = NULL;

int nCaveSpriteBank;

static int nSpriteAddressMask;

struct CaveSprite {
	char flip;
	char palette;
	char priority;
	int x; int y;
	int xsize; int ysize;
	int xzoom; int yzoom;
	int address;
};

static CaveSprite* pSpriteList = NULL;

int (*CaveSpriteBuffer)();

static unsigned char* pRow;
static unsigned char* pPixel;
static unsigned int* pSpriteData;
static unsigned int* pSpritePalette;

static unsigned short* pZBuffer = NULL;
static unsigned short* pZRow;
static unsigned short* pZPixel;

static int nPositionShift;

static int nSpriteRow, nSpriteRowSize;
static int nXPos, nYPos, nZPos;
static int nXSize, nYSize;
static int nSpriteXZoomSize, nSpriteYZoomSize;
static int nSpriteXOffset, nSpriteYOffset;

static int nFirstSprite[4], nLastSprite[4];

static int nTopSprite;
static int nZOffset;

typedef void (*RenderSpriteFunction)();
static RenderSpriteFunction* RenderSprite;

static int nLastBPP = 0;

// Include the sprite rendering functions
#include "cave_sprite_func.h"

// Donpachi/DoDonpachi sprite format (no zooming)
int CaveSpriteBuffer_NoZoom()
{
	unsigned short* pSprite = (unsigned short*)(CaveSpriteRAM + (nCaveSpriteBank << 14));
	CaveSprite* pBuffer = pSpriteList;
	int nPriority;
	
	nFirstSprite[0] = 0x00010000;
	nFirstSprite[1] = 0x00010000;
	nFirstSprite[2] = 0x00010000;
	nFirstSprite[3] = 0x00010000;
	
	nLastSprite[0] = -1;
	nLastSprite[1] = -1;
	nLastSprite[2] = -1;
	nLastSprite[3] = -1;

	short word;
	int x, y, xs, ys;
	
	for (int i = 0, z = 0; i < 0x0400; i++, pSprite += 8) {
					
		word = pSprite[4];

		xs = (word >> 8) & 0x01F;
		ys = word & 0x1F;
		if (ys == 0 || xs == 0) {
			continue;
		}
		xs <<= 4;
		ys <<= 4;

		x = (pSprite[2] + nCaveExtraXOffset) & 0x03FF;
		if (x >= 320) {
			if (x + xs <= 0x0400) {
				continue;
			}
		}

		y = pSprite[3] & 0x03FF;
		if (y >= 240) {
			if (y + ys <= 0x0400) {
				continue;
			}
		}

		// Sprite is both active and onscreen, so add it to the buffer

		word = pSprite[0];
		
		nPriority = (word >> 4) & 0x03;
		if (nLastSprite[nPriority] == -1) {
			nFirstSprite[nPriority] = z;
		}
		nLastSprite[nPriority] = z;
		
		pBuffer->priority = 8 >> nPriority;

		pBuffer->flip = (word >> 2) & 0x03;
		pBuffer->palette = (word >> 8) & 0x3F;

		pBuffer->address = pSprite[1] | ((word & 3) << 16);

		pBuffer->x = x;
		pBuffer->y = y;

		pBuffer->xsize = xs;
		pBuffer->ysize = ys;
		
		pBuffer++;
		z++;
	}

	return 0;
}

// Normal sprite format (zooming)
int CaveSpriteBuffer_Zoom()
{
	unsigned short* pSprite = (unsigned short*)(CaveSpriteRAM + (nCaveSpriteBank << 14));
	CaveSprite* pBuffer = pSpriteList;
	int nPriority;

	nFirstSprite[0] = 0x00010000;
	nFirstSprite[1] = 0x00010000;
	nFirstSprite[2] = 0x00010000;
	nFirstSprite[3] = 0x00010000;
	
	nLastSprite[0] = -1;
	nLastSprite[1] = -1;
	nLastSprite[2] = -1;
	nLastSprite[3] = -1;

	short word;
	int x, y, xs, ys;
	
	for (int i = 0, z = 0; i < 0x0400; i++, pSprite += 8) {
					
		word = pSprite[6];

		xs = (word >> 8) & 0x01F;
		ys = word & 0x1F;
		if (ys == 0 || xs == 0) {
			continue;
		}
		xs <<= 4;
		ys <<= 4;

		word = pSprite[2];

		nPriority = (word >> 4) & 0x03;

		x = ((pSprite[0] >> nPositionShift) + nCaveExtraXOffset) & 0x03FF;
		y = (pSprite[1] >> nPositionShift) & 0x03FF;
		
		if (pSprite[4] <= 0x0100 && pSprite[5] <= 0x0100) {
			if (x >= 320) {
				if (x + xs <= 0x0400) {
					continue;
				}
			}
			if (y >= 240) {
				if (y + ys <= 0x0400) {
					continue;
				}
			}
		}
		
		// Sprite is active and most likely on screen, so add it to the buffer

		if (nLastSprite[nPriority] == -1) {
			nFirstSprite[nPriority] = z;
		}
		nLastSprite[nPriority] = z;
		
		pBuffer->priority = 8 >> nPriority;
		
		pBuffer->xzoom = pSprite[4];
		pBuffer->yzoom = pSprite[5];	

		pBuffer->xsize = xs;
		pBuffer->ysize = ys;
		
		pBuffer->x = x;
		pBuffer->y = y;

		pBuffer->flip = (word >> 2) & 0x03;
		pBuffer->palette = (word >> 8) & 0x3F;

		pBuffer->address = pSprite[3] | ((word & 3) << 16);
		
		pBuffer++;
		z++;
	}

	return 0;
}

int CaveSpriteRender(int nLowPriority, int nHighPriority)
{
	static int nMaskLeft, nMaskRight, nMaskTop, nMaskBottom;
	CaveSprite* pBuffer;

	int nPriorityMask = 0;
	int nMaxZPos = -1;
	int nCurrentZPos = 0x00010000;
	int nUseBuffer = 0x00010000;
	int nFunction;
		
	if (nLastBPP != nBurnBpp) {
		nLastBPP = nBurnBpp;

		RenderSprite = RenderSprite_ROT0[nBurnBpp - 2];
	}
	
	if (nLowPriority == 0) {
		nZPos = -1;
		nTopSprite = -1;

		nMaskLeft = nMaskTop = 9999;
		nMaskRight = nMaskBottom = -1;
	}
	
	if ((nBurnLayer & 1) == 0) {
		return 0;
	}

	if (nHighPriority < 3) {
		for (int i = nHighPriority + 1; i < 4; i++) {
			if (nUseBuffer > nFirstSprite[i]) {
				nUseBuffer = nFirstSprite[i];
			}
		}
	}

	for (int i = nLowPriority; i <= nHighPriority; i++) {
		if (nCurrentZPos > nFirstSprite[i]) {
			nCurrentZPos = nFirstSprite[i];
		}
		if (nMaxZPos < nLastSprite[i]) {
			nMaxZPos = nLastSprite[i];
		}
		nPriorityMask |= 8 >> i;
	}

	nPriorityMask &= nSpriteEnable;
	if (nPriorityMask == 0) {
		return 0;
	}
	
	for (pBuffer = pSpriteList + nCurrentZPos; nCurrentZPos <= nMaxZPos; pBuffer++, nCurrentZPos++) {

		if ((pBuffer->priority & nPriorityMask) == 0) {
			continue;
		}

		nXPos = pBuffer->x;
		nYPos = pBuffer->y;
		
		pSpriteData = (unsigned int*)(CaveSpriteROM + ((pBuffer->address << 8) & nSpriteAddressMask));
		pSpritePalette = CavePalette + (pBuffer->palette << 8);

		nXSize = pBuffer->xsize;
		nYSize = pBuffer->ysize;

		if (pBuffer->xzoom == 0x0100 && pBuffer->yzoom == 0x0100) {					// This sprite doesn't use zooming
			
			nSpriteRowSize = pBuffer->xsize >> 2;
			
			if (pBuffer->flip & 1) {												// Y Flip
				pSpriteData += nSpriteRowSize * (nYSize - 1);
				nSpriteRowSize = -nSpriteRowSize;
			}

			if (nYPos >= 0x0200) {
				nYPos -= 0x0400;
			}

			if (nYPos < 0) {
				pSpriteData += nSpriteRowSize * -nYPos;
				nYSize += nYPos;
				nYPos = 0;
			}

			if ((nYPos + nYSize) > 240) {
				nYSize -= (nYPos + nYSize) - 240;
			}

			if (nXPos >= 0x0200) {
				nXPos -= 0x0400;
			}

			if (nXPos < 0) {
				if ((pBuffer->flip & 2) == 0) {
					pSpriteData += (-nXPos >> 4) << 2;
				}
				nXSize -= -nXPos & 0xFFF0;
				nXPos += -nXPos & 0xFFF0;
			}

			if (nXPos + nXSize >= 320) {
				if (pBuffer->flip & 2) {
					pSpriteData += ((nXPos + nXSize - 320) >> 4) << 2;
				}
				nXSize -= (nXPos + nXSize - 320) & 0xFFF0;
			}

			pRow = pBurnDraw + (nYPos * nBurnPitch) + (nXPos * nBurnBpp);

			nFunction = (pBuffer->flip & 2) << 1;									// X Flip

			if (nTopSprite > nCurrentZPos) {										// Test ZBuffer
				if (nXPos < nMaskRight && (nXPos + nXSize) >= nMaskLeft && nYPos < nMaskBottom && (nYPos + nYSize) >= nMaskTop) {
					nFunction |= 1;
				}				
			}

			if (nUseBuffer < nCurrentZPos) {										// Write ZBuffer
				nFunction |= 2;

				if (nXPos < nMaskLeft) {
					nMaskLeft = nXPos;
				}
				if ((nXPos + nXSize) > nMaskRight) {
					nMaskRight = nXPos + nXSize;
				}
				if (nYPos < nMaskTop) {
					nMaskTop = nYPos;
				}
				if ((nYPos + nYSize) > nMaskBottom) {
					nMaskBottom = nYPos + nYSize;
				}
			}

			if (nFunction & 3) {
				pZRow = pZBuffer + (nYPos * 320) + nXPos;
				nZPos = nCurrentZPos + nZOffset;
			}

			nXSize = nXSize >> 2;

			RenderSprite[nFunction]();
		} else {																	// This sprite uses zooming
			nSpriteXZoomSize = 0x01000000;											// * zoom factor = size of each screen pixel

			nXSize *= pBuffer->xzoom << 8;
			nXSize &= 0xFFFF0000;													// Round to multiple of whole pixel
			if (nXSize < 0x00010000) {												// Make sure the sprite is at least one pixel wide
				nXSize = 0x00010000;
			} else {
				nSpriteXZoomSize /= pBuffer->xzoom;
			}
			if (nSpriteXZoomSize > (pBuffer->xsize << 16)) {
				nSpriteXZoomSize = pBuffer->xsize << 16;
			}
			nSpriteXOffset = nSpriteXZoomSize >> 1;									// Make certain the pixels displayed are centered

			if (pBuffer->flip & 2) {												// X Flip
				nXPos += ((pBuffer->xsize << 16) - nXSize) >> 16;
			
				nSpriteXOffset = (pBuffer->xsize << 16) - nSpriteXOffset;
				nSpriteXZoomSize = -nSpriteXZoomSize;
			}

			if (nXPos >= 0x0200) {
				nXPos -= 0x0400;
			}

			if (nXPos < 0) {
				if ((nXPos + (nXSize >> 16)) <= 0) {
					continue;
				}
				nXPos = -nXPos;
				nSpriteXOffset += nXPos * nSpriteXZoomSize;
				nXSize -= nXPos << 16;
				nXPos = 0;
			}

			if ((nXPos + (nXSize >> 16)) >= 320) {
				if (nXPos >= 320) {
					continue;
				}
				nXSize = (320 - nXPos) << 16;
			}

			nSpriteRowSize = pBuffer->xsize;										// Size of each sprite row in memory
			nSpriteYZoomSize = 0x01000000;											// * zoom factor = size of each screen pixel

			nYSize *= pBuffer->yzoom << 8;
			nYSize &= 0xFFFF0000;													// Round to multiple of whole pixel
			if (nYSize < 0x00010000) {												// Make certain the sprite is at least one pixel high
				nYSize = 0x00010000;
			} else {
				nSpriteYZoomSize /= pBuffer->yzoom;
			}
			if (nSpriteYZoomSize > (pBuffer->ysize << 16)) {
				nSpriteYZoomSize = pBuffer->ysize << 16;
			}
			nSpriteYOffset = nSpriteYZoomSize >> 1;									// Make certain the pixels displayed are centered

			if (pBuffer->flip & 1) {												// Y Flip
				nYPos += ((pBuffer->ysize << 16) - nYSize) >> 16;
			
				nSpriteYOffset = (pBuffer->ysize << 16) - nSpriteYOffset;
				nSpriteYZoomSize = -nSpriteYZoomSize;
			}

			if (nYPos >= 0x0200) {
				nYPos -= 0x0400;
			}

			if (nYPos < 0) {
				if ((nYPos + (nYSize >> 16)) <= 0) {
					continue;
				}
				nYPos = -nYPos;
				nSpriteYOffset += nYPos * nSpriteYZoomSize;
				nYSize -= nYPos << 16;
				nYPos = 0;
			}

			if ((nYPos + (nYSize >> 16)) >= 240) {
				if (nYPos >= 240) {
					continue;
				}
				nYSize = (240 - nYPos) << 16;
			}

			pRow = pBurnDraw + (nYPos * nBurnPitch) + (nXPos * nBurnBpp);

			nFunction = 8;
			
			if (pBuffer->xzoom > 0x0100 || pBuffer->yzoom > 0x0100) {
				nFunction += 4;
			}

			if (nTopSprite > nCurrentZPos) {										// Test ZBuffer			
				if (nXPos < nMaskRight && (nXPos + (nXSize >> 16)) >= nMaskLeft && nYPos < nMaskBottom && (nYPos + (nYSize >> 16)) >= nMaskTop) {
					nFunction |= 1;
				}
			}

			if (nUseBuffer < nCurrentZPos) {										// Write ZBuffer
				nFunction += 2;			
				
				if (nXPos < nMaskLeft) {
					nMaskLeft = nXPos;
				}
				if ((nXPos + (nXSize >> 16)) > nMaskRight) {
					nMaskRight = nXPos + (nXSize >> 16);
				}
				if (nYPos < nMaskTop) {
					nMaskTop = nYPos;
				}
				if ((nYPos + (nYSize >> 16)) > nMaskBottom) {
					nMaskBottom = nYPos + (nYSize >> 16);
				}
			}

			if (nFunction & 3) {
				pZRow = pZBuffer + (nYPos * 320) + nXPos;
				nZPos = nCurrentZPos + nZOffset;
			}

			RenderSprite[nFunction]();
		}
	}

	if (nMaxZPos > nTopSprite) {
		nTopSprite = nMaxZPos;
	}

	if (nHighPriority == 3) {
		if (nZPos >= 0) {
			nZOffset += nTopSprite;
			if (nZOffset > 0xFC00) {
				memset(pZBuffer, 0, 320 * 240 * sizeof(short));
				nZOffset = 0;
			}
		}
	}
	
	return 0;
}

void CaveSpriteExit()
{
	free(pSpriteList);
	pSpriteList = NULL;

	free(pZBuffer);
	pZBuffer = NULL;

	return;
}

int CaveSpriteInit(int nType, int nROMSize)
{
	free(pSpriteList);
	pSpriteList = (CaveSprite*)malloc(0x0401 * sizeof(CaveSprite));
	if (pSpriteList == NULL) {
		CaveSpriteExit();
		return 1;
	}
	
	for (int i = 0; i < 0x0400; i++) {
		pSpriteList[i].xzoom = 0x0100;
		pSpriteList[i].yzoom = 0x0100;
	}
	for (int i = 0; i < 4; i++) {
		nFirstSprite[i] = 0x00010000;
		nLastSprite[i] = -1;
	}

	free(pZBuffer);
	pZBuffer = (unsigned short*)malloc(320 * 240 * sizeof(short));
	if (pZBuffer == NULL) {
		CaveSpriteExit();
		return 1;
	}

	memset(pZBuffer, 0, 320 * 240 * sizeof(short));
	nZOffset = 0;

	for (nSpriteAddressMask = 1; nSpriteAddressMask < nROMSize; nSpriteAddressMask <<= 1) {}
	nSpriteAddressMask--;

	if ((nType & 1) == 0) {
		CaveSpriteBuffer = &CaveSpriteBuffer_NoZoom;
	} else {
		CaveSpriteBuffer = &CaveSpriteBuffer_Zoom;
	}

	nPositionShift = 6;
	if (nType & 2) {
		nPositionShift = 0;
	}
	
	nCaveSpriteBank = 0;

	return 0;
}

