// Graphics format: 8  8 pixel tiles, 4/8 bits/pixel.

// Create a unique name for each of the functions
#define FN(a,b,c,d,e,f,g) RenderTile ## a ## _ROT ## b ## c ## d ## e ## f ## g
#define FUNCTIONNAME(a,b,c,d,e,f,g) FN(a,b,c,d,e,f,g)

#if ROT == 0

 #if XFLIP == 0
    #define ADVANCECOLUMN pPixel += (BPP >> 3)
 #elif XFLIP == 1
  #define ADVANCECOLUMN pPixel -= (BPP >> 3)
 #else
  #error illegal XFLIP value
 #endif

 #if YFLIP == 0
    #define ADVANCEROW pTileRow += ((BPP >> 3) * 320)
 #elif YFLIP == 1
  #define ADVANCEROW pTileRow -= ((BPP >> 3) * 320)
 #else
  #error illegal YFLIP value
 #endif

#elif ROT == 270

 #if XFLIP == 0
    #define ADVANCECOLUMN pPixel -= ((BPP >> 3) * 240)
 #elif XFLIP == 1
  #define ADVANCECOLUMN pPixel += ((BPP >> 3) * 240)
 #else
  #error illegal XFLIP value
 #endif

 #if YFLIP == 0
    #define ADVANCEROW pTileRow += (BPP >> 3)
 #elif YFLIP == 1
  #define ADVANCEROW pTileRow -= (BPP >> 3)
 #else
  #error illegal YFLIP value
 #endif

#else
 #error unsupported rotation angle specified
#endif

#if DOCLIP == 0
 #define CLIP _NOCLIP
 #if ROWSCROLL == 0
  #define TESTCLIP(x) 1
 #else
  #define TESTCLIP(x) nRowOffset >= (0 - x) && nRowOffset < (320 - x)
 #endif
#elif DOCLIP == 1
 #define CLIP _CLIP
 #if ROWSCROLL == 0
  #define TESTCLIP(x) nTileXPos >= (0 - x) && nTileXPos < (320 - x)
 #else
  #define TESTCLIP(x) nRowOffset >= (0 - x) && nRowOffset < (320 - x)
 #endif
#else
 #error illegal doclip value.
#endif

#if EIGHTBIT == 0
 #define DEPTH _16
#elif EIGHTBIT == 1
 #define DEPTH _256
#else
 #error illegal eightbit value
#endif

#define TESTCOLOUR(x) x

#if BPP == 16
 #define PLOTPIXEL(a,b) if (TESTCOLOUR(b) && TESTCLIP(a)) {			\
   	*((unsigned short*)pPixel) = (unsigned short)pTilePalette[b];	\
 }
#elif BPP == 24
 #define PLOTPIXEL(a,b) if (TESTCOLOUR(b) && TESTCLIP(a)) {			\
	unsigned int nRGB = pTilePalette[b];							\
	pPixel[0] = (unsigned char)nRGB;								\
	pPixel[1] = (unsigned char)(nRGB >> 8);							\
	pPixel[2] = (unsigned char)(nRGB >> 16);						\
 }
#elif BPP == 32
 #define PLOTPIXEL(a,b) if (TESTCOLOUR(b) && TESTCLIP(a)) {			\
	 *((unsigned int*)pPixel) = (unsigned int)pTilePalette[b];		\
 }
#else
 #error unsupported bitdepth specified.
#endif

// For decorating the function name
#if XFLIP == 1
 #if YFLIP == 1
  #define FLIP _FLIPXY
 #else
  #define FLIP _FLIPX
 #endif
#elif YFLIP == 1
 #define FLIP _FLIPY
#else
   #define FLIP _NOFLIP
#endif

#if ROWSELECT == 0
 #define SELECT _NOROWSELECT
#elif ROWSELECT == 1
 #define SELECT _ROWSELECT
#else
 #error unsupported rowselect mode specified.
#endif

#if ROWSCROLL == 0
 #define SCROLL _NOROWSCROLL
#elif ROWSCROLL == 1
 #define SCROLL _ROWSCROLL
#else
 #error unsupported rowscroll mode specified.
#endif

static void FUNCTIONNAME(BPP,ROT,FLIP,SCROLL,SELECT,CLIP,DEPTH)()
{
// Create an empty function if unsupported features are requested
#if ROT == 0 && XFLIP == 0 && YFLIP == 0 && !(ROWSCROLL == 1 && ROWSELECT == 1) && EIGHTBIT == 1
	
	unsigned char *pTileRow, *pPixel;
	int nColour;

 #if ROWSELECT == 0

	int y;

  #if ROT == 0
   #if YFLIP == 0
	for (y = 0, pTileRow = pTile; y < 8; y++, ADVANCEROW) {
   #else
	for (y = 7, pTileRow = pTile + ((BPP >> 3) * 320 * 7); y >= 0; y--, ADVANCEROW) {
   #endif
  #else
   #if YFLIP == 0
	for (y = 0, pTileRow = pTile; y < 8; y++, ADVANCEROW) {
   #else
	for (y = 7, pTileRow = pTile + ((BPP >> 3) * 7); y >= 0; y--, ADVANCEROW) {
   #endif
  #endif

  #if DOCLIP == 1
		if (nTileYPos + y < 0 || nTileYPos + y >= 240) {
   #if EIGHTBIT == 0
			pTileData++;
   #else
			pTileData += 2;
   #endif
			continue;
		}
  #endif

 #else
	pTileRow = pTile;
 #endif

 #if ROT == 0
  #if XFLIP == 0
   #define OFFSET(a) a
		pPixel = pTileRow;
  #else
   #define OFFSET(a) (7 - a)
		pPixel = pTileRow + ((BPP >> 3) * 7);
  #endif
 #else
  #if XFLIP == 0
   #define OFFSET(a) a
		pPixel = pTileRow;
  #else
   #define OFFSET(a) (7 - a)
		pPixel = pTileRow - ((BPP >> 3) * 240 * 7);
  #endif
 #endif

 #if ROWSCROLL == 1
		nRowOffset = (nTileXPos - pTileRowInfo[y]) & 0x01FF;
		if (nRowOffset >= 0x01F8) {
			nRowOffset -= 0x0200;
		}
		if (nRowOffset >= 320) {
  #if EIGHTBIT == 0
			pTileData++;
  #else
			pTileData += 2;
  #endif
			continue;
		}
		pPixel += (BPP >> 3) * nRowOffset;
 #endif

 #if EIGHTBIT == 0
		nColour = *pTileData++;
		PLOTPIXEL(OFFSET(0), nColour & 0x0F);
		ADVANCECOLUMN;
		nColour >>= 4;
		PLOTPIXEL(OFFSET(1), nColour & 0x0F);
		ADVANCECOLUMN;

		nColour >>= 4;
		PLOTPIXEL(OFFSET(2), nColour & 0x0F);
		ADVANCECOLUMN;
		nColour >>= 4;
		PLOTPIXEL(OFFSET(3), nColour & 0x0F);
		ADVANCECOLUMN;

		nColour >>= 4;
		PLOTPIXEL(OFFSET(4), nColour & 0x0F);
		ADVANCECOLUMN;
		nColour >>= 4;
		PLOTPIXEL(OFFSET(5), nColour & 0x0F);
		ADVANCECOLUMN;

		nColour >>= 4;
		PLOTPIXEL(OFFSET(6), nColour & 0x0F);
		ADVANCECOLUMN;
		nColour >>= 4;
		PLOTPIXEL(OFFSET(7), nColour & 0x0F);
 #else
		nColour = *pTileData++;
		PLOTPIXEL(OFFSET(0), nColour & 0xFF);
		ADVANCECOLUMN;
		nColour >>= 8;
		PLOTPIXEL(OFFSET(1), nColour & 0xFF);
		ADVANCECOLUMN;

		nColour >>= 8;
		PLOTPIXEL(OFFSET(2), nColour & 0xFF);
		ADVANCECOLUMN;
		nColour >>= 8;
		PLOTPIXEL(OFFSET(3), nColour & 0xFF);
		ADVANCECOLUMN;
		
		nColour = *pTileData++;
		PLOTPIXEL(OFFSET(4), nColour & 0xFF);
		ADVANCECOLUMN;
		nColour >>= 8;
		PLOTPIXEL(OFFSET(5), nColour & 0xFF);
		ADVANCECOLUMN;

		nColour >>= 8;
		PLOTPIXEL(OFFSET(6), nColour & 0xFF);
		ADVANCECOLUMN;
		nColour >>= 8;
		PLOTPIXEL(OFFSET(7), nColour & 0xFF);
 #endif

 #undef OFFSET

 #if ROWSELECT == 0
	}
 #endif

#endif
}

#undef FLIP
#undef CLIP
#undef PLOTPIXEL
#undef TESTCLIP
#undef TESTCOLOUR
#undef ADVANCEROW
#undef ADVANCECOLUMN
#undef SCROLL
#undef SELECT
#undef DEPTH
#undef FUNCTIONNAME
#undef FN

