/*************************************************************************

  Rainbow Islands C-Chip Protection

  2002-Feb rewritten by Robert Gallagher
	
  notes moved to header for clarity
************************************************************************/

#include "rastan.h"

static unsigned char* CRAM[8];

static int current_bank;

/*************************************
 *
 * C-Chip bank data
 *
 *************************************/

static void request_round_data(void)
{
	int round = CRAM[1][0x141]; /* 0...49 */

	memcpy(CRAM[1], CROM_BANK1, sizeof CROM_BANK1);
	memcpy(CRAM[2], CROM_BANK2, sizeof CROM_BANK2);

	CRAM[1][1] = cchip_round_height[round] >> 0;
	CRAM[1][2] = cchip_round_height[round] >> 8;

	CRAM[1][0x142] = cchip_round_address[round] >> 24;
	CRAM[1][0x143] = cchip_round_address[round] >> 16;
	CRAM[1][0x144] = cchip_round_address[round] >> 8;
	CRAM[1][0x145] = cchip_round_address[round] >> 0;

	/* set the secret room or boss flag */

	CRAM[1][0x148] = (round >= 40 || round % 4 == 3);
}

static void request_world_data(void)
{
	int world = CRAM[0][0x00D] / 4; /* 0...9 */

	/* first two bytes in each bank are left unchanged  */

	memcpy(CRAM[4] + 2, CROM_BANK4[world].data, CROM_BANK4[world].size);
	memcpy(CRAM[5] + 2, CROM_BANK5[world].data, CROM_BANK5[world].size);
	memcpy(CRAM[7] + 2, CROM_BANK7[world].data, CROM_BANK7[world].size);
	memcpy(CRAM[6] + 2, CROM_BANK6, sizeof CROM_BANK6);

}

static void request_goalin_data(void)
{
	int n = rand() % 15;

	CRAM[1][0x14B] = 0x00; /* x coordinates */
	CRAM[1][0x14D] = 0x10;
	CRAM[1][0x14F] = 0x20;
	CRAM[1][0x151] = 0x38;
	CRAM[1][0x153] = 0x50;
	CRAM[1][0x155] = 0x60;

	CRAM[1][0x14A] = cchip_goalin[n][0]; /* y coordinates */
	CRAM[1][0x14C] = cchip_goalin[n][1];
	CRAM[1][0x14E] = cchip_goalin[n][2];
	CRAM[1][0x150] = cchip_goalin[n][3];
	CRAM[1][0x152] = cchip_goalin[n][4];
	CRAM[1][0x154] = cchip_goalin[n][5];
}

void cchip_timer()
{
	if (CRAM[1][0x100] == 1)
	{
		request_round_data();

		CRAM[1][0x100] = 0xFF;
	}

	if (CRAM[5][0x000] == 1)
	{
		request_world_data();

		CRAM[5][0x000] = 0xFF;
	}

	if (CRAM[1][0x149] == 1)
	{
		request_goalin_data();

		CRAM[1][0x149] = 0xFF;
	}
/* todo
	coin_lockout_w(1, CRAM[0][8] & 0x80);
	coin_lockout_w(0, CRAM[0][8] & 0x40);
	coin_counter_w(1, CRAM[0][8] & 0x20);
	coin_counter_w(0, CRAM[0][8] & 0x10);
*/
	CRAM[0][3] = rainbowInput[0];
	CRAM[0][4] = rainbowInput[1];
	CRAM[0][5] = rainbowInput[2];
	CRAM[0][6] = rainbowInput[3];

}

/*************************************
 *
 * C-Chip reads and writes
 *
 *************************************/

void rainbow_cchip_bank_w(unsigned char d)
{
//	printf("rainbow_cchip_bank_w %x\n",d);
	current_bank = d & 7;
}

void rainbow_cchip_ram_w(unsigned int a,unsigned char d) 
{
//	printf("rainbow_cchip_ram_w %x to %x in bank %x\n",d,a,current_bank);
	CRAM[current_bank][a] = d;
}

unsigned char rainbow_cchip_ctrl_r() 
{
//	printf("rainbow_cchip_ctrl_r\n");
	return 1;
}

unsigned char rainbow_cchip_ram_r (unsigned int a) 
{
//	printf("rainbow_cchip_ram_r %x bank %x\n",a,current_bank);
	return CRAM[current_bank][a];
}

/*************************************
 *
 * C-Chip initialization
 *
 *************************************/

void rainbow_cchip_init()
{

	for (int i = 0; i < 8; i++)
	{
		CRAM[i] = (unsigned char*)malloc(0x400);
		memset(CRAM[i],0xFF,0x400);
	}

}

void rainbow_cchip_exit()
{
	for (int i = 0; i < 8; i++)
	{
		free(CRAM[i]);
	}
}

void rainbow_cchip_scan(int /*nAction*/)
{
	for (int i = 0; i < 8; i++)
	{
		SCAN_VAR(CRAM[i]);
	}
	SCAN_VAR(current_bank);
}


