// Burner Config file module
#include "app.h"

struct VidPresetData VidPreset[4] = {
	{ 400, 300},
	{ 640, 480},
	{ 1024, 768},
	{ 1280, 960},
};

static char szConfig[EXE_NAME_SIZE + 16];

static void CreateConfigName()
{
	memcpy(szConfig, "cfg\\", 4);
	memcpy(szConfig + 4, szAppExeName, strlen(szAppExeName));
	memcpy(szConfig + 4 + strlen(szAppExeName), ".ini", 5);

	return;
}

// Read in the config file for the whole application
int ConfigAppLoad()
{
	char szLine[256];
	FILE *h;

	CreateConfigName();

	if ((h = fopen(szConfig,"rt")) == NULL) {
		return 1;
	}

	// Go through each line of the config file
	while (fgets(szLine, sizeof(szLine), h)) {
		int nLen;

		nLen = strlen(szLine);
		
		// Get rid of the linefeed at the end
		if (szLine[nLen - 1] == 10) {
			szLine[nLen - 1] = 0;
			nLen--;
		}

#define VAR(x) { char *szValue = LabelCheck(szLine,#x);		\
  if (szValue) x = strtol(szValue, NULL, 0); }
#define FLT(x) { char *szValue = LabelCheck(szLine,#x);		\
  if (szValue) x = atof(szValue); }
#define STR(x) { char *szValue = LabelCheck(szLine,#x " ");	\
  if (szValue) strcpy(x,szValue); }

		// Video
		VAR(nUseSysMemory);
		VAR(nVidWidth); VAR(nVidHeight); VAR(nVidDepth); VAR(nVidRefresh);
		VAR(bVidArcaderes);
		VAR(VidPreset[0].nWidth); VAR(VidPreset[0].nHeight);
		VAR(VidPreset[1].nWidth); VAR(VidPreset[1].nHeight);
		VAR(VidPreset[2].nWidth); VAR(VidPreset[2].nHeight);
		VAR(VidPreset[3].nWidth); VAR(VidPreset[3].nHeight);
		VAR(nScreenSize);
		VAR(nWindowSize);
		VAR(bDoGamma);
		VAR(bVidUseHardwareGamma);
		VAR(bHardwareGammaOnly);
		FLT(nGamma);
		VAR(bVidFullStretch);
		VAR(bVidCorrectAspect);
		VAR(bVidTripleBuffer);
		VAR(bVidScanlines);
		VAR(nMonAspectX);
		VAR(nMonAspectY);
		VAR(bForce60Hz);

		VAR(nVidSelect);
		VAR(nVidBlitterOpt[0]);
		VAR(nVidBlitterOpt[1]);
		VAR(nVidBlitterOpt[2]);

		// DirectDraw blitter
		VAR(bVidScanHalf);
		
		// Direct3D blitter
		VAR(bVidBilinear);
		VAR(bVidScanDelay);
		VAR(bVidScanRotate);
		VAR(bVidScanBilinear);
		VAR(nVidScanIntensity);
		VAR(nVidFeedbackIntensity);
		VAR(nVidFeedbackOverSaturation);
		FLT(fVidScreenAngle);
		FLT(fVidScreenCurvature);
		VAR(bVidForce16bit);
		VAR(nVidTransferMethod);

		// Sound
		VAR(nAudSampleRate);
		VAR(nAudSegCount);
		VAR(bAppBassFilter);
		VAR(nInterpolation);

		// Other
		STR(szAppRomPaths[0]);
		STR(szAppRomPaths[1]);
		STR(szAppRomPaths[2]);
		STR(szAppRomPaths[3]);

		VAR(nVidSDisplayStatus);
		VAR(nMinChatFontSize);
		VAR(nMaxChatFontSize);
		
		VAR(bModelessMenu);

		VAR(bDrvSaveAll);
		VAR(nAppThreadPriority);
		VAR(bAlwaysProcessKeyboardInput);
		VAR(bAutoPause);
		VAR(bSaveInputs);

		VAR(nLoadMenuShowX);
		VAR(bLoadMenuAutoExpand);
		VAR(bAvbOnly);

		// Default Controls
		VAR(nPlayerDefaultControls[0]);
		STR(szPlayerDefaultIni[0]);
		VAR(nPlayerDefaultControls[1]);
		STR(szPlayerDefaultIni[1]);
		VAR(nPlayerDefaultControls[2]);
		STR(szPlayerDefaultIni[2]);
		VAR(nPlayerDefaultControls[3]);
		STR(szPlayerDefaultIni[3]);

#undef STR
#undef FLT
#undef VAR
	}

	fclose(h);
	return 0;
}

// Write out the config file for the whole application
int ConfigAppSave()
{
	FILE *h;

	if (bCmdOptUsed) {
		return 1;
	}

	CreateConfigName();

	if ((h = fopen(szConfig, "wt")) == NULL) {
		return 1;
	}

	// Write title
	fprintf(h,"// " APP_TITLE " v%s Config File\n\n", szAppBurnVer);
	fprintf(h,"// Don't edit this file manually unless you know what you're doing\n");
	fprintf(h,"// " APP_TITLE " will restore default settings when this file is deleted\n");

#define VAR(x) fprintf(h, #x " %d\n", x)
#define FLT(x) fprintf(h, #x " %f\n", x)
#define STR(x) fprintf(h, #x " %s\n", x)
	
	fprintf(h,"\n\n\n");
	fprintf(h,"// --- Video ----------------------------------------------------------------\n");

	fprintf(h,"\n// The display mode to use for fullscreen\n");
	VAR(nVidWidth); VAR(nVidHeight); VAR(nVidDepth);
	fprintf(h,"\n// Specify the refresh rate, 0 = default (changing this will not work with many video cards)\n");
	VAR(nVidRefresh);
	fprintf(h,"\n// If non-zero, use the same fullscreen resolution as the original arcade game\n");
	VAR(bVidArcaderes);
	fprintf(h,"\n// The preset resolutions appearing in the menu\n");
	VAR(VidPreset[0].nWidth); VAR(VidPreset[0].nHeight);
	VAR(VidPreset[1].nWidth); VAR(VidPreset[1].nHeight);
	VAR(VidPreset[2].nWidth); VAR(VidPreset[2].nHeight);
	VAR(VidPreset[3].nWidth); VAR(VidPreset[3].nHeight);
	fprintf(h,"\n// Full-screen size (0 = use display mode variables)\n");
	VAR(nScreenSize);	
	fprintf(h,"\n// Initial window size (0 = autosize)\n");
	VAR(nWindowSize);
	fprintf(h,"\n// If non-zero, perform gamma correction\n");
	VAR(bDoGamma);
	fprintf(h,"\n// If non-zero, use the video hardware to correct gamma\n");
	VAR(bVidUseHardwareGamma);
	fprintf(h,"\n// If non-zero, don't fall back on software gamma correction\n");
	VAR(bHardwareGammaOnly);
	fprintf(h,"\n// Gamma to correct with\n");
	FLT(nGamma);
	fprintf(h,"\n// If non-zero, allow stretching of the image to any size\n");
	VAR(bVidFullStretch);
	fprintf(h,"\n// If non-zero, stretch the image to the largest size preserving aspect ratio\n");
	VAR(bVidCorrectAspect);
	fprintf(h,"\n// If non-zero, try to use a triple buffer in fullscreen\n");
	VAR(bVidTripleBuffer);
	fprintf(h,"\n// If non-zero, draw scanlines to simulate a low-res monitor\n");
	VAR(bVidScanlines);
	fprintf(h,"\n// If non-zero, rotate scanlines and RGB effects for rotated games\n");
	VAR(bVidScanRotate);
	fprintf(h,"\n// The selected blitter module\n");
	VAR(nVidSelect);
	fprintf(h,"\n// Options for the blitter modules\n");
	VAR(nVidBlitterOpt[0]);
	VAR(nVidBlitterOpt[1]);
	VAR(nVidBlitterOpt[2]);
	fprintf(h,"\n// The aspect ratio of the monitor\n");
	VAR(nMonAspectX);
	VAR(nMonAspectY);
	fprintf(h,"\n// If non-zero, force all games to use a 60Hz refresh rate\n");
	VAR(bForce60Hz);
	
	fprintf(h,"\n// Buffering method: -1 = Autodetect best memory, 0 = try video memory, 1 = force system memory\n");
	VAR(nUseSysMemory);
	fprintf(h,"\n");
	fprintf(h,"// --- DirectDraw blitter settings ------------------------------------------\n");
	fprintf(h,"\n// If non-zero, draw scanlines at 50%% intensity\n");
	VAR(bVidScanHalf);
	fprintf(h,"\n");
	fprintf(h,"// --- Direct3D blitter settings --------------------------------------------\n");
	fprintf(h,"\n// If non-zero, use bi-linear filtering to display the image\n");
	VAR(bVidBilinear);
	fprintf(h,"\n// If non-zero, simulate slow phosphors (feedback)\n");
	VAR(bVidScanDelay);
	fprintf(h,"\n// If non-zero, use bi-linear filtering for the scanlines\n");
	VAR(bVidScanBilinear);
	fprintf(h,"\n// Maximum scanline intensity\n");
	VAR(nVidScanIntensity);
	fprintf(h,"\n// Feedback amount for slow phosphor simulation\n");
	VAR(nVidFeedbackIntensity);
	fprintf(h,"\n// Oversaturation amount for slow phosphor simulation\n");
	VAR(nVidFeedbackOverSaturation);
	fprintf(h,"\n// Angle at wich the emulated screen is tilted (in radians)\n");
	FLT(fVidScreenAngle);
	fprintf(h,"\n// Angle of the sphere segment used for the 3D screen (in radians)\n");
	FLT(fVidScreenCurvature);
	fprintf(h,"\n// If non-zero, force 16 bit emulation even in 32-bit screenmodes\n");
	VAR(bVidForce16bit);
	fprintf(h,"\n// If non-zero, force transfer of the image to video memory before transferring it to the texture\n");
	VAR(nVidTransferMethod);

	fprintf(h,"\n\n\n");
	fprintf(h,"// --- Sound ----------------------------------------------------------------\n");
	
	fprintf(h,"\n// Sample rate\n");
	VAR(nAudSampleRate);
	fprintf(h,"\n// Number of frames in sound buffer (= sound lag)\n");
	VAR(nAudSegCount);
	fprintf(h,"\n// If non-zero, filter the sound output to add more bass\n");
	VAR(bAppBassFilter);
	fprintf(h,"\n// The order of PCM/ADPCM interpolation\n");
	VAR(nInterpolation);

	fprintf(h,"\n\n\n");
	fprintf(h,"// --- UI -------------------------------------------------------------------\n");
	
	fprintf(h,"\n// 1 = display pause/record/replay/kaillera icons in the upper right corner of the display\n");
	VAR(nVidSDisplayStatus);
	fprintf(h,"\n// Minimum height (in pixels) of the font used for the Kaillera chat function.(used for arcade resolution)\n");
	VAR(nMinChatFontSize);
	fprintf(h,"\n// Maximum height (in pixels) of the font used for the Kaillera chat function (used for 1280x960 or higher).\n");
	VAR(nMaxChatFontSize);
	
	fprintf(h,"\n// Make the menu modeless\n");
	VAR(bModelessMenu);
	
	fprintf(h,"\n// If non-zero, load and save all ram (the state)\n");
	VAR(bDrvSaveAll);
	fprintf(h,"\n// The thread priority for the application. Do *NOT* edit this manually\n");
	VAR(nAppThreadPriority);	
	fprintf(h,"\n// If non-zero, process keyboard input even when the application loses focus\n");
	VAR(bAlwaysProcessKeyboardInput);
	fprintf(h,"\n// If non-zero, pause when the application loses focus\n");
	VAR(bAutoPause);
	fprintf(h,"\n// If non-zero, save the inputs for each game\n");
	VAR(bSaveInputs);
	fprintf(h,"\n");
	fprintf(h,"// --- Load Game Dialog -----------------------------------------------------\n");
	fprintf(h,"\n// Which systems to show ROMs for.\n");
	VAR(nLoadMenuShowX);
	fprintf(h,"\n// If non-zero, show all clones\n");
	VAR(bLoadMenuAutoExpand);
	fprintf(h,"\n// If non-zero, show only available ROMsets\n");
	VAR(bAvbOnly);

	fprintf(h,"\n// The paths to search for rom zips. (include trailing backslash)\n");
	STR(szAppRomPaths[0]);
	STR(szAppRomPaths[1]);
	STR(szAppRomPaths[2]);
	STR(szAppRomPaths[3]);

	fprintf(h,"\n\n\n");
	fprintf(h,"// --- miscellaneous -------------------------------------------------------\n");
	
	fprintf(h, "\n// Player default controls, number is the index of the configuration in the input dialog\n");
	VAR(nPlayerDefaultControls[0]);
	STR(szPlayerDefaultIni[0]);
	VAR(nPlayerDefaultControls[1]);
	STR(szPlayerDefaultIni[1]);
	VAR(nPlayerDefaultControls[2]);
	STR(szPlayerDefaultIni[2]);
	VAR(nPlayerDefaultControls[3]);
	STR(szPlayerDefaultIni[3]);

	fprintf(h,"\n\n\n");

#undef STR
#undef FLT
#undef VAR

	fclose(h);
	return 0;
}

char* LabelCheck(char *s,char *szLabel)
{
	int nLen;
	if (s == NULL) {
		return NULL;
	}
	if (szLabel == NULL) {
		return NULL;
	}
	nLen = strlen(szLabel);
	
	SKIP_WS(s);													// Skip whitespace

	if (strncmp(s, szLabel, nLen)){								// Doesn't match
		return NULL;
	}
	return s + nLen;
}

int QuoteRead(char** pszQuote, char** pszEnd, char* szSrc)		// Read a quoted string from szSrc and point to the end
{
	static char szQuote[QUOTE_MAX];
	char *s = szSrc;
	int i;
	
	SKIP_WS(s);													// Skip whitespace

	if (*s != '\"')	{											// Didn't start with a quote
		return 1;
	}
	s++;

	// Copy the characters into szQuote
	for (i = 0; i < QUOTE_MAX - 1; i++, s++) {
		if (*s == '\"') {
			s++;
			break;
		}
		if (*s == 0) {											// Stop copying if end of line or end of string
			break;
		}
		szQuote[i] = *s;										// Copy character to szQuote
	}
	szQuote[i] = 0;												// Zero-terminate szQuote

	if (pszQuote) {
		*pszQuote = szQuote;
	}
	if (pszEnd)	{
		*pszEnd = s;
	}
	
	return 0;
}

