// Burner data file module (for ROM managers)
// Original version LoqiqX
// August 2002 - modified to use the parent identifer from the BurnDriver struct and to support boardROMs

#include "app.h"

int nDatFileType;

int write_datfile(int nDatType)
{
	char cFilename[80];
	FILE *fDat=0;
	int nRet=0;
	int nOldSelect=0;
	int nGameSelect=0;
	int nParentSelect=0;

	if (nDatType==0)
		sprintf(cFilename, APP_TITLE " v%.20s (CMPro).dat", szAppBurnVer);
	else
		sprintf(cFilename, APP_TITLE " v%.20s (RomCenter).dat", szAppBurnVer);

	if ((fDat=fopen(cFilename, "w"))==0)
		return -1;

	nOldSelect=nBurnDrvSelect; // preserve the currently selected driver

	if (nDatType==0)
	{
		fprintf(fDat, "clrmamepro (\n");
		fprintf(fDat, "\tname \"" APP_TITLE "\"\n");
		fprintf(fDat, "\tdescription \"" APP_TITLE " v%.20s\"\n", szAppBurnVer);
		fprintf(fDat, "\tcategory \"" APP_DESCRIPTION "\"\n");
		fprintf(fDat, "\tversion %s\n", szAppBurnVer);
		fprintf(fDat, "\tauthor \"" APP_TITLE " v%.20s\"\n", szAppBurnVer);
		fprintf(fDat, "\tforcezipping zip\n");
		fprintf(fDat, ")\n\n");
	}
	else
	{
		fprintf(fDat, "[CREDITS]\n");
		fprintf(fDat, "Author=" APP_TITLE "\n");
		fprintf(fDat, "Version=%.20s\n", szAppBurnVer);
		fprintf(fDat, "Comment=" APP_DESCRIPTION "\n");
		fprintf(fDat, "[DAT]\n");
		fprintf(fDat, "version=2.00\n");
		fprintf(fDat, "[EMULATOR]\n");
		fprintf(fDat, "refname=" APP_TITLE "\n");
		fprintf(fDat, "version=" APP_TITLE " v%.20s\n", szAppBurnVer);
		fprintf(fDat, "[GAMES]\n");
	}

	for (nGameSelect=0;(unsigned int)nGameSelect<nBurnDrvCount;nGameSelect++)
	{
		char sgName[16];
		char spName[16];
		int nRet=0;
		unsigned int i=0;

		nBurnDrvSelect=nGameSelect; // Switch to driver nGameSelect
		
		if (BurnDrvGetFlags() & BDF_BOARDROM) {
			continue;
		}
		
		strcpy(sgName, BurnDrvText(0));
		strcpy(spName, ""); // make sure this string is empty before we start

		// Check to see if the game has a parent
		if (BurnDrvText(6))
		{
			strcpy(spName, BurnDrvText(6));
			nParentSelect=-1;
			for (i=0;i<nBurnDrvCount;i++)
			{
				nBurnDrvSelect=i;
				if (!strcmp(spName, BurnDrvText(0)))
				{
					nParentSelect=i;
				}
			}
		
			nBurnDrvSelect=nGameSelect; // restore driver select
		}
		else
			nParentSelect=nGameSelect;
			
		if (nDatType==0)
		{
			if (nParentSelect==-1)
			{
				fprintf(fDat, "# Missing parent %s. Someone should add it to " APP_TITLE "!\n\n", spName);
			}

			fprintf(fDat, "game (\n");
			fprintf(fDat, "\tname %s\n", sgName);
			fprintf(fDat, "\tdescription \"%s", BurnDrvText(1));
			if (BurnDrvText(2))
			{
				if (strlen(BurnDrvText(2)) > 0)
				{
					fprintf(fDat, " [%s]", BurnDrvText(2));
				}
			}
			fprintf(fDat, "\"\n");
			fprintf(fDat, "\tyear %s\n", BurnDrvText(5));
			fprintf(fDat, "\tmanufacturer \"%s\"\n", BurnDrvText(3));

			if (nParentSelect!=nGameSelect && nParentSelect!=-1)
			{
				fprintf(fDat, "\tcloneof %s\n", spName);
				fprintf(fDat, "\tromof %s\n", spName);
			}
			else
			{
				// Add "romof" (but not 'cloneof') line for games that have boardROMs
				if (BurnDrvText(7))
				{
					fprintf(fDat, "\tromof %s\n", BurnDrvText(7));
				}
			}
		}

		for (i=0; nRet==0 && i<0x100; i++) // up to 0x100 ROMs in one game
		{
			int nRetTmp=0;
			struct BurnRomInfo ri;
			int nLen; unsigned int nCrc;
			char *szPossibleName=NULL;
			int j, nMerged=0, nTest=2;

			memset(&ri,0,sizeof(ri));

			nRet=BurnDrvGetRomInfo(&ri,i);
			nRet+=BurnDrvGetRomName(&szPossibleName,i,0);

			if (ri.nLen==0) continue;

			if (nRet==0)
			{
				while (nTest--)
				{
					nLen=ri.nLen; nCrc=ri.nCrc;
					nRetTmp=0;
					
					switch (nTest)
					{
						case 1:
						{
							char* szBoardROM;

							nBurnDrvSelect=nGameSelect;
							szBoardROM=BurnDrvText(7);

							if (szBoardROM==NULL)
							{
								continue;
							}

							// Find boardROMs
							for (nBurnDrvSelect=0; nBurnDrvSelect<nBurnDrvCount; nBurnDrvSelect++)
							{
								if (!strcmp(szBoardROM, BurnDrvText(0)))
								{
									break;
								}
							}
							break;
						}
						
						case 0:
						{
							if (nParentSelect!=nGameSelect && nParentSelect!=-1)
							{
								nBurnDrvSelect=nParentSelect;	// Switch to parent
							}
							else
							{
								continue;
							}
							break;
						}

						default:
						{
							nTest = 0;
							continue;
						}
					}
					
					for (j=0; nRetTmp==0 && j<0x100; j++)		// up to 0x100 ROMs in one game
					{
						struct BurnRomInfo riTmp;
						char *szPossibleNameTmp=NULL;

						memset(&riTmp,0,sizeof(riTmp));

						nRetTmp=BurnDrvGetRomInfo(&riTmp,j);
						nRetTmp+=BurnDrvGetRomName(&szPossibleNameTmp,j,0);

						if (riTmp.nLen==0) nRetTmp++;

						if (nRetTmp==0)
						{
							if (riTmp.nCrc==nCrc && !strcmp(szPossibleName, szPossibleNameTmp))
							{
								nMerged++;
								nRetTmp++;
							}
						}
					}
				}

				nBurnDrvSelect=nGameSelect;					// Switch back to game
				
				if (nDatType==0)
				{
					if (!nMerged)
						fprintf(fDat, "\trom ( name %s size %d crc %08x )\n", szPossibleName, ri.nLen, ri.nCrc);
					else
						fprintf(fDat, "\trom ( name %s merge %s size %d crc %08x )\n", szPossibleName, szPossibleName, ri.nLen, ri.nCrc);
				}
				else
				{
					if (nParentSelect!=nGameSelect && nParentSelect!=-1)
					{
						nBurnDrvSelect=nParentSelect; // Switch to parent
						fprintf(fDat, "%s%s", spName, BurnDrvText(1));
						nBurnDrvSelect=nGameSelect; // Switch back to game
					}
					else
						fprintf(fDat, "%s%s", BurnDrvText(0), BurnDrvText(1));

					fprintf(fDat, "%s%s", BurnDrvText(0), BurnDrvText(1));

		   			fprintf(fDat, "%s%08x%d", szPossibleName, ri.nCrc, ri.nLen);

					if (nParentSelect!=nGameSelect && nParentSelect!=-1)
					{
						fprintf(fDat, "%s", spName);
					}
					else
					{
						// force the neogeo bios merging details
						if (BurnDrvText(7))
							fprintf(fDat, "%s", BurnDrvText(7));
					}

					if (!nMerged)
						fprintf(fDat, "\n");
					else
						fprintf(fDat, "%s\n", szPossibleName);
				}
			}
		}

		if (nDatType==0)
			fprintf(fDat, ")\n\n");
	}
	
	for (nBurnDrvSelect=0; nBurnDrvSelect<nBurnDrvCount; nBurnDrvSelect++)
	{
		int i;
		
		if (!(BurnDrvGetFlags() & BDF_BOARDROM)) {
			continue;
		}

		if (nDatType==0)
		{
			fprintf(fDat, "resource (\n");
			fprintf(fDat, "\tname %s\n", BurnDrvText(0));
			fprintf(fDat, "\tdescription \"%s", BurnDrvText(1));
			if (BurnDrvText(2))
			{
				if (strlen(BurnDrvText(2)) > 0)
				{
					fprintf(fDat, " [%s]", BurnDrvText(2));
				}
			}
			fprintf(fDat, "\"\n");
			fprintf(fDat, "\tyear %s\n", BurnDrvText(5));
			fprintf(fDat, "\tmanufacturer \"%s\"\n", BurnDrvText(3));

		}

		for (i=0; i<0x100; i++)		// up to 0x100 ROMs in one game
		{
			struct BurnRomInfo ri;
			char *szPossibleName=NULL;

			memset(&ri,0,sizeof(ri));

			nRet=BurnDrvGetRomInfo(&ri,i);
			nRet+=BurnDrvGetRomName(&szPossibleName,i,0);

			if (ri.nLen==0) continue;

			if (nRet==0) {
				if (nDatType==0)
				{
					fprintf(fDat, "\trom ( name %s size %d crc %08x )\n", szPossibleName, ri.nLen, ri.nCrc);
				}
				else
				{
					fprintf(fDat, "%s%s", BurnDrvText(0), BurnDrvText(1));
					fprintf(fDat, "%s%s", BurnDrvText(0), BurnDrvText(1));
		   			fprintf(fDat, "%s%08x%d", szPossibleName, ri.nCrc, ri.nLen);
					fprintf(fDat, "\n");
				}
			}
		}
		
		if (nDatType==0)
		{
			fprintf(fDat, ")\n");
		}
	}

	nBurnDrvSelect=nOldSelect;

	fclose(fDat);

	return nRet;
};
