// Burner Game Control
#include "app.h"

static char szPlay[4][4]={"p1 ", "p2 ", "p3 ", "p4 "};

#define KEY(x) { pgi->nInput = 2; pgi->Input.Switch.nCode = (unsigned short)(x); }
#define MACRO(x) { pgi->Macro.nMode = 1; pgi->Macro.Switch.nCode = (unsigned short)(x); }

// Configure the misc game controls
int GamcMisc(struct GameInp* pgi, char* szi, int nPlayer)
{
	switch (nPlayer) {
		case 0:
			// Set general controls according to Player 1 settings
			if (strcmp(szi, "diag") == 0) {
				KEY(DIK_F2);
				return 0;
			}
			if (strcmp(szi, "reset") == 0) {
				KEY(DIK_F3);
				return 0;
			}
			if (strcmp(szi, "service" ) == 0) {
				KEY(DIK_9);
				return 0;
			}

			// Player 1 controls
			if (strcmp(szi, "p1 start") == 0) {
				KEY(DIK_1);
				return 0;
			}
			if (strcmp(szi, "p1 select" ) == 0) {
				KEY(DIK_3);
				return 0;
			}
			if (strcmp(szi, "p1 coin" ) == 0) {
				KEY(DIK_5);
				return 0;
			}
			break;
		case 1:
			if (strcmp(szi, "p2 start") == 0) {
				KEY(DIK_2);
				return 0;
			}
			if (strcmp(szi, "p2 select" ) == 0) {
				KEY(DIK_4);
				return 0;
			}
			if (strcmp(szi, "p2 coin" ) == 0) {
				KEY(DIK_6);
				return 0;
			}
			break;
		case 2:
			if (strcmp(szi, "p3 coin" ) == 0) {
				KEY(DIK_7);
				return 0;
			}	
			if (strcmp(szi, "p3 start") == 0) {
				KEY(DIK_3);
				return 0;
			}
			break;
		case 3:
			if (strcmp(szi, "p4 start") == 0) {
				KEY(DIK_4);
				return 0;
			}
			if (strcmp(szi, "p4 coin" ) == 0) {
				KEY(DIK_8);
				return 0;
			}
			break;
	}
	
	return 0;
}

static void SetSliderKey(struct GameInp* pgi, int k0, int k1, int nSlide)
{
	pgi->nInput = 6;
	pgi->Input.Slider.SliderAxis.nSlider[0] = (unsigned char)k0;
	pgi->Input.Slider.SliderAxis.nSlider[1] = (unsigned char)k1;

	if (nSlide == 2) {
		// Sliding
		pgi->Input.Slider.nSliderSpeed = 0x0700;
		pgi->Input.Slider.nSliderCenter = 0;
	} else {
		// Sliding (centering)
		pgi->Input.Slider.nSliderSpeed = 0x0E00;
		pgi->Input.Slider.nSliderCenter = 10;
	}
}

int GamcAnalogKey(struct GameInp* pgi, char* szi, int nPlayer, int nSlide)
{
	char *szSearch = NULL;
	int k0 = 0, k1 = 0;
	szSearch = szPlay[nPlayer & 3];
	if (strnicmp(szSearch, szi, 3) != 0) {				// Not our player
		return 1;
	}
	szi += 3;

	if (*szi == 0) {
		return 1;
	}
	if (strcmp(szi + 1, "-axis") != 0 && strcmp(szi + 1, "-dial") != 0) {
		return 1;
	}

	// Keyboard
	if (szi[0] == 'x') {
		k0 = DIK_LEFT;
		k1 = DIK_RIGHT;
	}
	if (szi[0] == 'y') {
		k0 = DIK_UP;
		k1 = DIK_DOWN;
	}
	if (szi[0] == 'z') {
		k0 = DIK_V;
		k1 = DIK_F;
	}
	if (k0 == 0 || k1 == 0) {
		return 1;
	}

	SetSliderKey(pgi, k0, k1, nSlide);

	return 0;
}

int GamcAnalogJoy(struct GameInp* pgi, char* szi, int nPlayer, int nJoy, int nSlide)
{
	int nAxis = 0;
	
	char* szSearch = szPlay[nPlayer & 3];
	if (strnicmp(szSearch, szi, 3) != 0)	{	// Not our player
		return 1;
	}
	szi += 3;

	if (szi[0] == 0) {
		return 1;
	}
	if (strncmp(szi + 1, "-axis", 5) != 0 && strncmp(szi + 1, "-dial", 5) != 0) {
		return 1;
	}

	if (strncmp(szi, "x", 1) == 0) {
		nAxis = 0;
	}
	if (strncmp(szi, "y", 1) == 0) {
		nAxis = 1;
	}
	if (strncmp(szi, "z", 1) == 0) {
		nAxis = 2;
	}

	if (strlen(szi) > 6) {
		if (strcmp(&szi[6], "-lo") == 0) {
			nSlide = 3;
		}
		if (strcmp(&szi[6], "-hi") == 0) {
			nSlide = 4;
		}
	}

	switch (nSlide) {
		case 2:								// Sliding
			pgi->nInput = 7;
			pgi->Input.Slider.nSliderValue = 0x8000;		// Put slider in the middle
			pgi->Input.Slider.nSliderSpeed = 0x0700;
			pgi->Input.Slider.nSliderCenter = 0;
			pgi->Input.Slider.JoyAxis.nAxis = nAxis;
			pgi->Input.Slider.JoyAxis.nJoy = (unsigned char)nJoy;
			break;
		case 1:								// Sliding (centering)
			pgi->nInput = 7;
			pgi->Input.Slider.nSliderValue = 0x8000;		// Put slider in the middle
			pgi->Input.Slider.nSliderSpeed = 0x0E00;
			pgi->Input.Slider.nSliderCenter = 10;
			pgi->Input.Slider.JoyAxis.nAxis = nAxis;
			pgi->Input.Slider.JoyAxis.nJoy = (unsigned char)nJoy;
			break;
		case 3:								// Absolute, axis-hi
			pgi->nInput = 4;
			pgi->Input.JoyAxis.nAxis = nAxis;
			pgi->Input.JoyAxis.nJoy = (unsigned char)nJoy;
			break;
		case 4:								// Absolute, axis-lo
			pgi->nInput = 5;
			pgi->Input.JoyAxis.nAxis = nAxis;
			pgi->Input.JoyAxis.nJoy = (unsigned char)nJoy;
			break;
		default:							// Absolute, entire axis
			pgi->nInput = 3;
			pgi->Input.JoyAxis.nAxis = nAxis;
			pgi->Input.JoyAxis.nJoy = (unsigned char)nJoy;
	}

	return 0;
}

// Set a Game Input to use Device 'nDevice' if it belongs to 'nPlayer'
// -2 = nothing  -1 == keyboard, 0 == joystick 1, 1 == joystick 2 etc...
int GamcPlayer(struct GameInp* pgi, char* szi, int nPlayer, int nDevice)
{
	char *szSearch = NULL;
	int nJoyBase = 0;
	szSearch = szPlay[nPlayer & 3];
	if (strnicmp(szSearch, szi, 3) != 0) {	// Not our player
		return 1;
	}
	szi += 3;

	if (nDevice <= -2) {
		int bOurs = 0;
		if (strcmp(szi, "up") == 0 || strcmp(szi, "y-axis-lo") == 0) {
			bOurs = 1;
		}
		if (strcmp(szi, "down") == 0 || strcmp(szi, "y-axis-hi") == 0) {
			bOurs = 1;
		}
		if (strcmp(szi, "left") == 0 || strcmp(szi, "x-axis-lo") == 0) {
			bOurs = 1;
		}
		if (strcmp(szi, "right") == 0 || strcmp(szi, "x-axis-hi") == 0) {
			bOurs = 1;
		}
		if (strncmp(szi, "fire ", 5) == 0) {
			bOurs = 1;
		}

		if (!bOurs)	{
			return 1;
		}

		pgi->nInput = 1;					// Constant zero
		pgi->Input.Constant.nConst = 0;		//

		return 0;
	}
	
	// Now check the rest of it
	if (nDevice == -1) {
		// Keyboard
		if (strcmp(szi, "up") == 0 || strcmp(szi, "y-axis-lo") == 0) {
			KEY(DIK_UPARROW);
		}
		if (strcmp(szi, "down") == 0 || strcmp(szi, "y-axis-hi") == 0) {
			KEY(DIK_DOWNARROW);
		}
		if (strcmp(szi, "left") == 0 || strcmp(szi, "x-axis-lo") == 0) {
			KEY(DIK_LEFTARROW);
		}
		if (strcmp(szi, "right") == 0 || strcmp(szi, "x-axis-hi") == 0) {
			KEY(DIK_RIGHTARROW);
		}
		if (nFireButtons == 4) {
			if (strcmp(szi, "fire 1") == 0) {
				KEY(DIK_Z);
			}
			if (strcmp(szi, "fire 2") == 0) {
				KEY(DIK_X);
			}
			if (strcmp(szi, "fire 3") == 0) {
				KEY(DIK_C);
			}
			if (strcmp(szi, "fire 4") == 0) {
				KEY(DIK_V);
			}
		} else {
			if (bStreetFighterLayout) {
				if (strcmp(szi, "fire 1") == 0) {
					KEY(DIK_A);
				}
				if (strcmp(szi, "fire 2") == 0) {
					KEY(DIK_S);
				}
				if (strcmp(szi, "fire 3") == 0) {
					KEY(DIK_D);
				}
				if (strcmp(szi, "fire 4") == 0) {
					KEY(DIK_Z);
				}
				if (strcmp(szi, "fire 5") == 0) {
					KEY(DIK_X);
				}
				if (strcmp(szi, "fire 6") == 0) {
					KEY(DIK_C);
				}
				if (stricmp(szi, "3 Punch") == 0) {
					MACRO(DIK_F);
				}
				if (stricmp(szi, "3 Kick") == 0) {
					MACRO(DIK_V);
				}
			} else {
				if (strcmp(szi, "fire 1") == 0) {
					KEY(DIK_Z);
				}
				if (strcmp(szi, "fire 2") == 0) {
					KEY(DIK_X);
				}
				if (strcmp(szi, "fire 3") == 0) {
					KEY(DIK_C);
				}
				if (strcmp(szi, "fire 4") == 0) {
					KEY(DIK_A);
				}
				if (strcmp(szi, "fire 5") == 0) {
					KEY(DIK_S);
				}
				if (strcmp(szi, "fire 6") == 0) {
					KEY(DIK_D);
				}
			}
		}
		
		return 0;
	}

	// Joystick
	nJoyBase = 0x4000;
	nJoyBase |= nDevice << 8;

	if (strcmp(szi, "up") == 0)	{
		KEY(nJoyBase + 0x02)
	}
	if (strcmp(szi, "down") == 0) {
		KEY(nJoyBase + 0x03)
	}
	if (strcmp(szi, "left") == 0)	{
		KEY(nJoyBase + 0x00)
	}
	if (strcmp(szi, "right") == 0) {
		KEY(nJoyBase + 0x01)
	}
	if (strncmp(szi, "fire ", 5) == 0) {	
		char *szb = szi + 5;
		int nButton;
		nButton = strtol(szb, NULL, 0);
		if (nButton >= 1) {
			nButton--;
		}
		KEY(nJoyBase + 0x80 + nButton);
	}
	
	return 0;
}

int GamcPlayerHotRod(struct GameInp* pgi, char* szi, int nPlayer, int nFlags, int nSlide)
{
	char *szSearch = NULL;
	szSearch = szPlay[nPlayer & 3];
	int k0 = 0, k1 = 0;
	
	if (strnicmp(szSearch, szi, 3) != 0) {					// Not our player
		return 1;
	}
	szi += 3;

	if ((nFlags & 1) == 0) {
		
		// X-Arcade / Hanaho HotRod joystick left side
		if (strcmp(szi, "x-axis") == 0 || strcmp(szi, "x-dial") == 0) {
			k0 = DIK_NUMPAD4;
			k1 = DIK_NUMPAD6;
		}
		if (strcmp(szi, "y-axis") == 0 || strcmp(szi, "y-dial") == 0) {
			k0 = DIK_NUMPAD8;
			k1 = DIK_NUMPAD2;
		}
		
		if (strcmp(szi, "up") == 0 || strcmp(szi, "y-axis-lo") == 0) {
			KEY(DIK_NUMPAD8);
		}
		if (strcmp(szi, "down") == 0 || strcmp(szi, "y-axis-hi") == 0) {
			KEY(DIK_NUMPAD2);
		}
		if (strcmp(szi, "left") == 0 || strcmp(szi, "x-axis-lo") == 0) {
			KEY(DIK_NUMPAD4);
		}
		if (strcmp(szi, "right") == 0 || strcmp(szi, "x-axis-hi") == 0) {
			KEY(DIK_NUMPAD6);
		}

		if (nFireButtons == 4) {
			if (strcmp(szi, "fire 1") == 0) {
				KEY(DIK_C);
			}
			if (strcmp(szi, "fire 2") == 0) {
				KEY(DIK_LSHIFT);
			}
			if (strcmp(szi, "fire 3") == 0) {
				KEY(DIK_Z);
			}
			if (strcmp(szi, "fire 4") == 0) {
				KEY(DIK_X);
			}
		} else {
			if (bStreetFighterLayout) {
				// Street Fighter and other Capcom 6-button games
				if (strcmp(szi, "fire 1") == 0) {
					KEY(DIK_LCONTROL);
				}
				if (strcmp(szi, "fire 2") == 0) {
					KEY(DIK_LALT);
				}
				if (strcmp(szi, "fire 3") == 0) {
					KEY(DIK_SPACE);
				}
				if (strcmp(szi, "fire 4") == 0) {
					KEY(DIK_LSHIFT);
				}
				if (strcmp(szi, "fire 5") == 0) {
					KEY(DIK_Z);
				}
				if (strcmp(szi, "fire 6") == 0) {
					KEY(DIK_X);
				}
				
				// Map the 3x macros on X-Arcade
				if ((nFlags & 0x10)) {
					if (stricmp(szi, "3 Punch") == 0) {
						MACRO(DIK_C);
					}
					if (stricmp(szi, "3 Kick") == 0) {
						MACRO(DIK_5);
					}
				}
			} else {
				if (strcmp(szi, "fire 1") == 0) {
					KEY(DIK_LSHIFT);
				}
				if (strcmp(szi, "fire 2") == 0) {
					KEY(DIK_Z);
				}
				if (strcmp(szi, "fire 3") == 0) {
					KEY(DIK_X);
				}
				if (strcmp(szi, "fire 4") == 0) {
					KEY(DIK_LCONTROL);
				}
				if (strcmp(szi, "fire 5") == 0) {
					KEY(DIK_LALT);
				}
				if (strcmp(szi, "fire 6") == 0) {
					KEY(DIK_SPACE);
				}
			}
		}
		
		// Map remaining buttons as usual
		if (strcmp(szi, "start") == 0) {
			KEY(DIK_1);
		}
		if (strcmp(szi, "coin") == 0) {
			KEY(DIK_3);
		}
		
		if (k0 && k1) {
			SetSliderKey(pgi, k0, k1, nSlide);
		}

		return 0;

	} else {
		if ((nFlags & 1) == 1) {
			// X-Arcade / Hanaho HotRod joystick right side

			if (strcmp(szi, "x-axis") == 0 || strcmp(szi, "x-dial") == 0) {
				k0 = DIK_D;
				k1 = DIK_G;
			}
			if (strcmp(szi, "y-axis") == 0 || strcmp(szi, "y-dial") == 0) {
				k0 = DIK_R;
				k1 = DIK_F;
			}

			if (strcmp(szi, "up") == 0 || strcmp(szi, "y-axis-lo") == 0)	{
				KEY(DIK_R);
			}
			if (strcmp(szi, "down") == 0 || strcmp(szi, "y-axis-hi") == 0) {
				KEY(DIK_F);
			}
			if (strcmp(szi, "left") == 0 || strcmp(szi, "x-axis-lo") == 0) {
				KEY(DIK_D);
			}
			if (strcmp(szi, "right") == 0 || strcmp(szi, "x-axis-hi") == 0) {
				KEY(DIK_G);
			}

			if (nFireButtons == 4) {
				if (strcmp(szi, "fire 1") == 0) {
					KEY(DIK_RBRACKET);
				}
				if (strcmp(szi, "fire 2") == 0) {
					KEY(DIK_W);
				}
				if (strcmp(szi, "fire 3") == 0) {
					KEY(DIK_E);
				}
				if (strcmp(szi, "fire 4") == 0) {
					KEY(DIK_LBRACKET);
				}
			} else {
				if (bStreetFighterLayout) {
					// Street Fighter and other Capcom 6-button games
					if (strcmp(szi, "fire 1") == 0) {
						KEY(DIK_A);
					}
					if (strcmp(szi, "fire 2") == 0) {
						KEY(DIK_S);
					}
					if (strcmp(szi, "fire 3") == 0) {
						KEY(DIK_Q);
					}
					if (strcmp(szi, "fire 4") == 0) {
						KEY(DIK_W);
					}
					if (strcmp(szi, "fire 5") == 0) {
						KEY(DIK_E);
					}
					if (strcmp(szi, "fire 6") == 0) {
						KEY(DIK_LBRACKET);
					}
				
					// Map the 3x macros on X-Arcade
					if ((nFlags & 0x10)) {
						if (stricmp(szi, "3 Punch") == 0) {
							MACRO(DIK_RBRACKET);
						}
						if (stricmp(szi, "3 Kick") == 0) {
							MACRO(DIK_6);
						}
					}
				} else {				
					if (strcmp(szi, "fire 1") == 0) {
						KEY(DIK_W);
					}
					if (strcmp(szi, "fire 2") == 0) {
						KEY(DIK_E);
					}
					if (strcmp(szi, "fire 3") == 0) {
						KEY(DIK_LBRACKET);
					}
					if (strcmp(szi, "fire 4") == 0) {
						KEY(DIK_A);
					}
					if (strcmp(szi, "fire 5") == 0) {
						KEY(DIK_S);
					}
					if (strcmp(szi, "fire 6") == 0) {
						KEY(DIK_Q);
					}
				}
			}

			// Map remaining buttons as usual
			if (strcmp(szi, "start") == 0) {
				KEY(DIK_2);
			}
			if (strcmp(szi, "coin") == 0) {
				KEY(DIK_4);
			}

			if (k0 && k1) {
				SetSliderKey(pgi, k0, k1, nSlide);
			}

			return 0;
		}
	}

	return 1;												// Couldn't map input
}

#undef MACRO
#undef KEY

