//---------------------------------------------------------------------------
// Burner - Arcade emulator interface
// Copyright (c) 2001 Dave (www.finalburn.com), all rights reserved.

// This refers to all code except where stated otherwise
// (e.g. unzip and zlib code)

// You can use, modify and redistribute this code freely as long as you
// don't do so commercially. This copyright notice must remain with the code.
// If your program uses this code, you must either distribute or link to the
// source code. If you modify or improve this code, you must distribute
// the source code improvements.

// Dave
// Homepage: www.finalburn.com
// E-mail:  dave@finalburn.com
//---------------------------------------------------------------------------

// Main module
#include "app.h"

#ifdef _MSC_VER
  #include <winable.h>
 #ifdef _DEBUG
  #include <crtdbg.h>
 #endif
#endif

#include "kailleraclient.h"

HINSTANCE hUser32;

// #define APP_DEBUG_LOG			// log debug messages to zzd.txt

HINSTANCE hAppInst = NULL;			// Application Instance
HANDLE hMainThread;
long int nMainThreadID;
HACCEL hAccel = NULL;
static char *szCmdLine = "";
int nAppVirtualFps = 6000;			// App fps * 100
int nUseSysMemory = -1;
int bAppBassFilter = 0;
int nMonAspectX = 4;
int nMonAspectY = 3;
int nAppShowCmd = 0;
int nAppThreadPriority = THREAD_PRIORITY_NORMAL;
char szAppBurnVer[16] = "";
char szAppExeName[EXE_NAME_SIZE + 1];
bool bCmdOptUsed = 0;
bool bAlwaysProcessKeyboardInput = false;

static bool bNumlockStatus;

// Used for the load/save dialog in commdlg.h (savestates, input replay, wave logging)
char szChoice[MAX_PATH] = "";
OPENFILENAME ofn;

bool SetNumLock(bool bState)
{
	char keyState[256];

	GetKeyboardState((LPBYTE)&keyState);
	if ((bState && !(keyState[VK_NUMLOCK] & 1)) || (!bState && (keyState[VK_NUMLOCK] & 1))) {
		keybd_event(VK_NUMLOCK, 0, KEYEVENTF_EXTENDEDKEY, 0 );

		keybd_event(VK_NUMLOCK, 0, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, 0);
	}

	return keyState[VK_NUMLOCK] & 1;
}

// Show a message box with an error message
int AppError(char *szText, int bWarning)
{
	AudSoundStop();
	MessageBox(hScrnWnd, szText, (bWarning ? APP_TITLE " Warning" : APP_TITLE " Error"), MB_OK | (bWarning ? MB_ICONWARNING : MB_ICONERROR));
	AudSoundPlay();
	return 0;
}

static int AppInit()
{

#ifdef _DEBUG
	AllocConsole();
#endif

	// Create a handle to the main thread of execution
	DuplicateHandle(GetCurrentProcess(), GetCurrentThread(), GetCurrentProcess(), &hMainThread, 0, false, DUPLICATE_SAME_ACCESS);

	kailleraInit();

	// Load config for the application
	if (ConfigAppLoad()) {
		ConfigAppSave();			// create initial config file
	}

	// Set the thread priority for the main thread
	SetThreadPriority(GetCurrentThread(), nAppThreadPriority);

	bCheatsAllowed = true;

	// Init the Burn library
	BurnLibInit();
	
	ComputeGammaLUT();
	MediaInit();

	//ayeye
	hAccel = LoadAccelerators(hAppInst, MAKEINTRESOURCE(IDR_ACCELERATOR));

	// Build the ROM information
	CreateROMInfo();

	bNumlockStatus = SetNumLock(false);

	return 0;
}

static int AppExit()
{
	SetNumLock(bNumlockStatus);

	kailleraShutdown();
	DrvExit();						// Make sure any game driver is exitted
	FreeROMInfo();
	MediaExit(true);
	BurnLibExit();					// Exit the Burn library
	ConfigAppSave();				// Save config for the application

	if (hAccel) {
		DestroyAcceleratorTable(hAccel);
		hAccel = NULL;
	}

	FreeLibrary(hUser32);

	CloseHandle(hMainThread);

	return 0;
}

int AppMessage(MSG *pMsg)
{
	if (IsDialogMessage(hInpdDlg, pMsg)) return 0;
	if (IsDialogMessage(hInpCheatDlg, pMsg)) return 0;
	if (IsDialogMessage(hInpDIPSWDlg, pMsg)) return 0;
	
	if (IsDialogMessage(hInpsDlg, pMsg)) return 0;
	if (IsDialogMessage(hInpcDlg, pMsg)) return 0;

	return 1; // Didn't process this message
}

bool AppProcessKeyboardInput()
{
	bool bProcessInput = true;
	
	if (hwndChat) {
		return false;
	}

	if (bAlwaysProcessKeyboardInput) {
		HWND hActive = GetActiveWindow();
		bProcessInput = (hActive && (hActive == hScrnWnd || hActive == hInpdDlg || hActive == hInpCheatDlg || hActive == hInpdDlg || hActive == hInpDIPSWDlg));
	}

	return bProcessInput;
}

#ifdef APP_DEBUG_LOG
static FILE *DebugFile = NULL;

// Debug printf to a file
static int __cdecl AppDebugPrintf(char *szFormat, ...)
{
	va_list Arg;
	va_start(Arg, szFormat);

	if (DebugFile) {
		vfprintf(DebugFile, szFormat, Arg);
		fflush(DebugFile);
	}

	va_end(Arg);
	return 0;
}
#endif

int ProcessCmdLine()
{
	unsigned int i;
	int nOptX = 0, nOptY = 0, nOptD = 0;
	int nOpt1Size;
	char szOpt2[2] = "";
	char szName[MAX_PATH];

	if (szCmdLine[0] == '\"') {
		int nLen = strlen(szCmdLine);
		nOpt1Size = 1;
		while (szCmdLine[nOpt1Size] != '\"' && nOpt1Size < nLen) {
			nOpt1Size++;
		}
		if (nOpt1Size == nLen) {
			szName[0] = 0;
		} else {
			nOpt1Size++;
			strncpy(szName, szCmdLine + 1, nOpt1Size - 2);
			szName[nOpt1Size - 2] = 0;
		}
	} else {
		int nLen = strlen(szCmdLine);
		nOpt1Size = 0;
		while (szCmdLine[nOpt1Size] != ' ' && nOpt1Size < nLen) {
			nOpt1Size++;
		}
		strncpy(szName, szCmdLine, nOpt1Size);
		szName[nOpt1Size] = 0;
	}
	
	sscanf(&szCmdLine[nOpt1Size], "%2s %i x %i x %i", szOpt2, &nOptX, &nOptY, &nOptD);

	if (strlen(szName)) {
		bool bFullscreen = 1;
		bCmdOptUsed = 1;

		if (strcmp(szOpt2, "-r") == 0) {
			if (nOptX && nOptY) {
				nVidWidth = nOptX;
				nVidHeight = nOptY;
			}
			if (nOptD) {
				nVidDepth = nOptD;
			}
		} else {
			if (strcmp(szOpt2, "-a") == 0) {
				bVidArcaderes = 1;
			} else {
				if (strcmp(szOpt2, "-w") == 0) {
					bCmdOptUsed = 0;
					bFullscreen = 0;
				}
			}
		}

		if (bFullscreen) {
			nVidFullscreen = 1;
		}

		if (strcmp(&szName[strlen(szName) - 3], ".fs") == 0) {
			if (StateLoad(szName, 1)) {
				return 1;
			} else {
				bRunPause = 1;
			}
		} else {
			if (strcmp(&szName[strlen(szName) - 3], ".fr") == 0) {
				if (StartReplay(szName)) {
					return 1;
				}
			} else {
				for (i = 0; i < nBurnDrvCount; i++) {
					nBurnDrvSelect = i;
					if (strcmp(BurnDrvText(0), szName) == 0) {
						DrvInit(i, true);				
						break;
					}
				}
				if (i == nBurnDrvCount) {
					char szErrorString[128];
					sprintf(szErrorString, "%s is not supported by FB Alpha.", szName);
					AppError(szErrorString, 0);
					return 1;
				}
			}
		}
	}

	if (bDrvOkay && nVidFullscreen) {
		PostMessage(NULL, WM_APP + 0, 0, 0);
	} else {
		MenuEnableItems();
	}
	
	return 0;
}

// Main program entry point
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR lpCmdLine, int nShowCmd)
{
	hAppInst = hInstance;
	nAppShowCmd = nShowCmd;

  	AppDirectory();								// Set current directory to be the applications directory

	// Make version string
	if (nBurnVer & 0xFF) {
		// private version (alpha)
		sprintf (szAppBurnVer, "%x.%x.%x.%x", nBurnVer >> 20, (nBurnVer >> 16) & 0x0F, (nBurnVer >> 8) & 0xFF, nBurnVer & 0xFF);
	} else {
		// public version
		sprintf (szAppBurnVer, "%x.%x.%x", nBurnVer >> 20, (nBurnVer >> 16) & 0x0F, (nBurnVer >> 8) & 0xFF);
	}

	// Make sure there are roms and cfg subdirectories
	CreateDirectory("roms", NULL);
	CreateDirectory("cfg", NULL);
	CreateDirectory("state", NULL);
	CreateDirectory("cheat", NULL);
	CreateDirectory("replay", NULL);
	CreateDirectory("sshots", NULL);
	CreateDirectory("wav", NULL);

#ifdef APP_DEBUG_LOG
	DebugFile = fopen("zzd.txt", "wt");			// Log debug to a file
	dprintf = AppDebugPrintf;					// Redirect Burn library debug to our function
#endif

	INITCOMMONCONTROLSEX initCC = {
		sizeof(INITCOMMONCONTROLSEX),
		ICC_BAR_CLASSES | ICC_COOL_CLASSES | ICC_LISTVIEW_CLASSES | ICC_PROGRESS_CLASS | ICC_TREEVIEW_CLASSES,
	};

	if (!InitCommonControlsEx(&initCC)) {
		char szErrorString[] = "Sorry, FB Alpha requires Windows 98 or later to run.";
		AppError(szErrorString, 0);

		return 0;
	}
	
	if (lpCmdLine) {
		szCmdLine = lpCmdLine;
	}

	if (!(AppInit())) {							// Init the application
		if (!(ProcessCmdLine())) {
			RunMessageLoop();					// Run the application message loop
		}
	}
	
	AppExit();									// Exit the application

#ifdef APP_DEBUG_LOG
	if (DebugFile) {
		fclose(DebugFile);
		DebugFile = NULL;
	}
#endif

#ifdef _MSC_VER
 #ifdef _DEBUG
	_CrtSetDbgFlag(_CRTDBG_LEAK_CHECK_DF);		// Check for memory leaks
 #endif
#endif

	return 0;
}
