// DirectD3D blitter

// #define USE_D3D_REFERENCE_DEVICE

#include "app.h"
#include "directx_support.h"

#ifdef _DEBUG
 #define PRINT_DEBUG_INFO
#endif

#define DIRECT3D_VERSION 0x0700							// Use this Direct3D version
#define D3D_OVERLOADS
#include <d3d.h>

const float PI = 3.14159265358979323846f;

typedef struct _D3DTLVERTEX2 {
	union { D3DVALUE sx; D3DVALUE dvSX; };
	union { D3DVALUE sy; D3DVALUE dvSY; };
	union { D3DVALUE sz; D3DVALUE dvSZ; };
	union { D3DVALUE rhw; D3DVALUE dvRHW; };
	union { D3DCOLOR color; D3DCOLOR dcColor; };
	union { D3DCOLOR specular; D3DCOLOR dcSpecular; };
	union { D3DVALUE tu; D3DVALUE dvTU; };
	union { D3DVALUE tv; D3DVALUE dvTV; };
	union { D3DVALUE tu1; D3DVALUE dvTU1; };
	union { D3DVALUE tv1; D3DVALUE dvTV1; };
#if(DIRECT3D_VERSION >= 0x0500)
 #if (defined __cplusplus) && (defined D3D_OVERLOADS)
	_D3DTLVERTEX2() { }
	_D3DTLVERTEX2(const D3DVECTOR& v, float _rhw, D3DCOLOR _color, D3DCOLOR _specular, float _tu, float _tv, float _tu1, float _tv1)
	{
		sx = v.x; sy = v.y; sz = v.z; rhw = _rhw;
		color = _color; specular = _specular;
		tu = _tu; tv = _tv;
		tu1 = _tu1; tv1 = _tv1;
	}
 #endif
#endif
} D3DTLVERTEX2, *LPD3DTLVERTEX2;

#define D3DFVF_TLVERTEX2 ( D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_SPECULAR | D3DFVF_TEX2 | D3DFVF_TEXCOORDSIZE2(0) | D3DFVF_TEXCOORDSIZE2(1) )

typedef struct _D3DLVERTEX2 {
	union { D3DVALUE x; D3DVALUE dvX; };
	union { D3DVALUE y; D3DVALUE dvY; };
	union { D3DVALUE z; D3DVALUE dvZ; };
    DWORD dwReserved;
	union { D3DCOLOR color; D3DCOLOR dcColor; };
	union { D3DCOLOR specular; D3DCOLOR dcSpecular; };
	union { D3DVALUE tu; D3DVALUE dvTU; };
	union { D3DVALUE tv; D3DVALUE dvTV; };
	union { D3DVALUE tu1; D3DVALUE dvTU1; };
	union { D3DVALUE tv1; D3DVALUE dvTV1; };
#if(DIRECT3D_VERSION >= 0x0500)
 #if (defined __cplusplus) && (defined D3D_OVERLOADS)
	_D3DLVERTEX2() { }
	_D3DLVERTEX2(const D3DVECTOR& v, D3DCOLOR _color, D3DCOLOR _specular, float _tu, float _tv, float _tu1, float _tv1)
	{
		x = v.x; y = v.y; z = v.z; dwReserved = 0;
		color = _color; specular = _specular;
		tu = _tu; tv = _tv;
		tu1 = _tu1; tv1 = _tv1;
	}
 #endif
#endif
} D3DLVERTEX2, *LPD3DLVERTEX2;

#define D3DFVF_LVERTEX2 ( D3DFVF_XYZ | D3DFVF_RESERVED1 | D3DFVF_DIFFUSE | D3DFVF_SPECULAR | D3DFVF_TEX2 | D3DFVF_TEXCOORDSIZE2(0) | D3DFVF_TEXCOORDSIZE2(1) )

static bool	bLostControl;

static bool bUsePageflip;			// use a flipping surface when in fullscreen mode
static bool bUseTriplebuffer;

static bool bMultiTexturing;
static bool bRenderToTexture;

struct EffectPreset {
	bool bUseFilter;
	bool bUseEffects;
	bool bUseScanlines;
	bool bUseFeedback;
	bool bUsePrescale;

	int nIntensity;
	int nFloor;
	int nSaturation;
	int nPatternMode;
	char* pPatternName;
	int nAttenuation;
	int nBlendMode;

	int nScanIntensity;

	int nAmount;
	int nOverSaturation;
};

static EffectPreset PresetInfo[] = {
	{ true, false, false, false, false, 0x00, 0x00, 0x00, 0, NULL,				 			0x00, 0, 0x00000000, 0, 0 },	// 1x zoom
	{ true, true,  true,  false, false, 0x80, 0x20, 0x00, 0, "4x4 Mini 1.rgb",				0x00, 0, 0x008F8F8F, 0, 0 },	// 2x zoom
	{ true, true,  true,  false, false, 0xA0, 0x30, 0x00, 2, "4x6 RGB Pattern.rgb",			0x00, 0, 0x005F5F5F, 0, 0 },	// 3x zoom
	{ true, true,  true,  false, true,  0x80, 0x20, 0x00, 3, "6x8 RGB Pattern.rgb",			0x20, 0, 0x004F4F4F, 0, 0 },	// 4x zoom
	{ true, true,  true,  false, true,  0x60, 0x10, 0x48, 3, "9x10 Ellipsoid.rgb",			0x20, 0, 0x005F5F5F, 0, 0 },	// 5x zoom
	{ true, false, false, false, false, 0x00, 0x00, 0x00, 0, NULL,				 			0x00, 0, 0x00000000, 0, 0 },	// 6x zoom
	{ true, false, false, false, false, 0x00, 0x00, 0x00, 0, NULL,				 			0x00, 0, 0x00000000, 0, 0 },	// 7x zoom
	{ true, false, false, false, false, 0x00, 0x00, 0x00, 0, NULL,				 			0x00, 0, 0x00000000, 0, 0 },	// 8x zoom

	// Selectable presets
	{ true, true, false, false, false, 0x0100, 0x80, 0x40, 2, "4x6 RGB Pattern.rgb",	    0,	  1, 0, 0, 0 },		// Small Faint RGB mask
	{ true, true, false, false, false, 0x0100, 0x80, 0x40, 2, "10x6 Large Dot.rgb",			0,	  1, 0, 0, 0 },		// Large RGB Dot mask
	{ true, true, false, false, false, 0x0100, 0x90, 0x80, 2, "12x10 Large Ellipsoid.rgb",	0,	  1, 0, 0, 0 },		// Large Faint RGB mask
	{ true, true, false, false, false, 0x0100, 0xC0, 0x00, 0, "4x4 MAME rgbtiny.rgb",	    0,	  1, 0, 0, 0 },		// MAME rgbtiny	small
	{ true, true, false, false, false, 0x0100, 0xD0, 0x00, 0, "8x8 MAME rgbtiny.rgb",	    0,	  1, 0, 0, 0 },		// MAME rgbtiny	large
	{ true, true, true , false, false, 0x0030, 0x00, 0x00, 0, "3x1 Aperture Grille.rgb",	0x20, 0, 0x00BFBFBF, 0, 0 },		// Aperture Grille
	{ true, true, false, false, false, 0x0040, 0x00, 0x00, 0, "10x6 Large Dot.rgb",			0x08, 0, 0, 0, 0 },		// Large Oversaturated RGB Dot
	{ true, true, false, false, false, 0x00E0, 0x00, 0x40, 3, "18x10 Large Round.rgb",		0x10, 0, 0, 0, 0 },		// Huge Oversaturated pattern
};

static bool bUseRGBEffects = false;						// Use RGB effects

static int nRGBScanlineIntensity = 0;					// Scanline intensity when RG effects are used

static int nRGBEffectIntensity = 0x0100;				// Multiply pattern RGB values with this
static int nRGBEffectFloor = 0x00;						// Minimum pattern colour grey-value
static int nRGBEffectSaturation = 0x00;					// Minimum saturation of the pattern colours
static int nRGBEffectPatternmode = 0;					// How to use the Floor/Saturation values
static int nRGBEffectAttenuation = 0;					// How much to attenuate the image before applying RGB effects
static char* pRGBEffectPatternName = "4x6 RGB Pattern.rgb";	// Pattern to use
static int nRGBEffectBlendmode = 1;						// Set of D3D blendmodes

static bool bRotateGame;
static bool bRotateEffects;								// Use rotated scanlines/RGB patterns for rotated games

static bool bUse3DProjection;

static float fPreviousScreenAngle;
static float fPreviousScreenCurvature;

static int nPreScale = 0;

static IDirectDraw7* pDD = NULL;						// DirectDraw interface
static IDirect3D7* pD3D = NULL;							// Direct3D interface
static IDirect3DDevice7* pD3DDevice = NULL;

static D3DDEVICEDESC7 d3dDeviceDesc;					// holds capabilities of the 3D hardware

static IDirectDrawSurface7* pPrimarySurf = NULL;		// Primary surface
static IDirectDrawSurface7* pBackbuffer = NULL;			// Back buffer surface

static D3DVIEWPORT7 Viewport = {0, 0, 0, 0, 0.0f, 1.0f};

static RECT Dest = { 0, 0, 0, 0 };

static int nGameWidth = 0, nGameHeight = 0;				// Game screen size
static int nDtosWidth = 0, nDtosHeight = 0;				// Emulator screen size
static int nScrnWidth, nScrnHeight;						// Screen size
static int nDtosDepth = 0, nDtosBpp = 0;
static int nScrnDepth;
static unsigned char* DtosMem = NULL;					// Memory buffer
static int nDtosMemPitch = 0;							// Memory buffer pitch
static IDirectDrawSurface7* pEmuImage[4] = {NULL, };	// The emulator screen surfaces/textures
static int nTextureWidth, nTextureHeight;
static IDirectDrawSurface7* pRGBEffectTexture = NULL;	// The RGB effects texture
static IDirectDrawSurface7* pScanlineTexture[2] = {NULL, };	// The scanline textures

static DWORD nScanlineOp;

static D3DTLVERTEX2 vScreen[4];							// Vectors for emulator image, screen coordinates
static D3DLVERTEX2* v3DScreen = NULL;					// Vectors for emulator image, 3D coordinates
static D3DTLVERTEX vFeedbackImage[4];					// Vectors for feedback

static D3DMATRIX matWorld;
static D3DMATRIX matView;
static D3DMATRIX matProjection;
// static D3DMATRIX matIdentity = D3DMATRIX(1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1);

static void SetWorldMatrix()
{
	matWorld._12 = matWorld._13 = matWorld._14 = matWorld._21 = matWorld._24 = 0.0f;
	matWorld._31 = matWorld._34 = matWorld._41 = matWorld._42 = matWorld._43 = 0.0f;
	matWorld._11 = matWorld._44 = 1.0f;
		
	matWorld._22 = (float)cos(fVidScreenAngle);
	matWorld._23 = (float)sin(fVidScreenAngle);
	matWorld._32 = -(float)sin(fVidScreenAngle);
	matWorld._33 = (float)cos(fVidScreenAngle);
		
	pD3DDevice->SetTransform(D3DTRANSFORMSTATE_WORLD, &matWorld);
}

static void SetViewMatrix(float fNearPlane)
{
	matView._12 = matView._13 = matView._14 = 0.0f;
	matView._21 = matView._23 = matView._24 = 0.0f;
	matView._31 = matView._32 = matView._34 = 0.0f;
	matView._41 = 0.0f;
	matView._11 = matView._22 = matView._33 = matView._44 = 1.0f;

	matView._42 = -1.0f;
	matView._43 = fNearPlane;
				
	pD3DDevice->SetTransform(D3DTRANSFORMSTATE_VIEW, &matView);
}

static void SetProjectionMatrix(float fNearPlane)
{
	// Set projection matrix for perspective
	memset(&matProjection, 0, sizeof(D3DMATRIX));

	float fFarPlane = 1000.0f;
	float Q = fFarPlane / (fFarPlane - fNearPlane);

	matProjection._11 = fNearPlane;
	matProjection._22 = fNearPlane;
	matProjection._33 = Q;
	matProjection._34 = 1.0f;
	matProjection._43 = -Q * fNearPlane;
			
	pD3DDevice->SetTransform(D3DTRANSFORMSTATE_PROJECTION, &matProjection);
}

static int Update3DScreen()
{
	if (bUse3DProjection) {
		
		if (v3DScreen == NULL) {
			v3DScreen = (D3DLVERTEX2*)malloc(6 * 16 * 16 * sizeof(D3DLVERTEX2));
			if (v3DScreen == NULL) {
				return 1;
			}
		}
		
		// Use a 3D world and tilt the image backward

		if (fVidScreenCurvature == 0.0f) {
			v3DScreen[0] = D3DLVERTEX2(D3DVECTOR(-1.0f, 2.0f, 0.0f), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), vScreen[0].tu, vScreen[0].tv, vScreen[0].tu1, vScreen[0].tv1);
			v3DScreen[1] = D3DLVERTEX2(D3DVECTOR( 1.0f, 2.0f, 0.0f), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), vScreen[1].tu, vScreen[1].tv, vScreen[1].tu1, vScreen[1].tv1);
			v3DScreen[2] = D3DLVERTEX2(D3DVECTOR(-1.0f, 0.0f, 0.0f), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), vScreen[2].tu, vScreen[2].tv, vScreen[2].tu1, vScreen[2].tv1);
			v3DScreen[3] = D3DLVERTEX2(D3DVECTOR( 1.0f, 0.0f, 0.0f), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), vScreen[3].tu, vScreen[3].tv, vScreen[3].tu1, vScreen[3].tv1);
		} else {
			int nXAspect = 0, nYAspect = 0;
			BurnDrvGetScreen(&nXAspect, &nYAspect);

			float nXAngle, nYAngle;

			if (nXAspect > nYAspect) {
				nXAngle = fVidScreenCurvature;
				nYAngle = fVidScreenCurvature * nYAspect / nXAspect;
			} else {
				nXAngle = fVidScreenCurvature * nXAspect / nYAspect;
				nYAngle = fVidScreenCurvature;
			}

			float nXFactor = 1.0f / (float)sin(nXAngle / 2);
			float nYFactor = 1.0f / (float)sin(nYAngle / 2);

			for (int y = 0; y < 16; y++) {

				float nTop = nYFactor * (float)sin(-nYAngle / 2 + nYAngle * y / 16);
				float nBottom = nYFactor * (float)sin(-nYAngle / 2 + nYAngle * (y + 1) / 16);

				float nTopFactor = (float)cos(-nYAngle / 2 + nYAngle * y / 16);
				float nBottomFactor = (float)cos(-nYAngle / 2 + nYAngle * (y + 1) / 16);
				
				for (int x = 0; x < 16; x++) {

					float nLeft = nXFactor * (float)sin(-nXAngle / 2 + nXAngle * x / 16);
					float nRight = nXFactor * (float)sin(-nXAngle / 2 + nXAngle * (x + 1) / 16);
					
					float nLeftFactor = (float)cos(-nXAngle / 2 + nXAngle * x / 16);
					float nRightFactor = (float)cos(-nXAngle / 2 + nXAngle * (x + 1) / 16);

					if (bRotateGame) {
						v3DScreen[6 * y + ((15 - x) * 6 * 16) + 0] = D3DLVERTEX2(D3DVECTOR(nTopFactor    * nLeft,  1.0f - nLeftFactor  * nTop,    1.0f - nTopFactor    * nLeftFactor ), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * y + ((15 - x) * 6 * 16) + 1] = D3DLVERTEX2(D3DVECTOR(nTopFactor    * nRight, 1.0f - nRightFactor * nTop,    1.0f - nTopFactor    * nRightFactor), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * y + ((15 - x) * 6 * 16) + 2] = D3DLVERTEX2(D3DVECTOR(nBottomFactor * nLeft,  1.0f - nLeftFactor  * nBottom, 1.0f - nBottomFactor * nLeftFactor ), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);

						v3DScreen[6 * y + ((15 - x) * 6 * 16) + 3] = D3DLVERTEX2(D3DVECTOR(nTopFactor    * nRight, 1.0f - nRightFactor * nTop,    1.0f - nTopFactor    * nRightFactor), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * y + ((15 - x) * 6 * 16) + 4] = D3DLVERTEX2(D3DVECTOR(nBottomFactor * nLeft,  1.0f - nLeftFactor  * nBottom, 1.0f - nBottomFactor * nLeftFactor ), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * y + ((15 - x) * 6 * 16) + 5] = D3DLVERTEX2(D3DVECTOR(nBottomFactor * nRight, 1.0f - nRightFactor * nBottom, 1.0f - nBottomFactor * nRightFactor), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
					} else {
						v3DScreen[6 * x + (y * 6 * 16) + 0] = D3DLVERTEX2(D3DVECTOR(nTopFactor    * nLeft,  1.0f - nLeftFactor  * nTop,    1.0f - nTopFactor    * nLeftFactor ), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * x + (y * 6 * 16) + 1] = D3DLVERTEX2(D3DVECTOR(nTopFactor    * nRight, 1.0f - nRightFactor * nTop,    1.0f - nTopFactor    * nRightFactor), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * x + (y * 6 * 16) + 2] = D3DLVERTEX2(D3DVECTOR(nBottomFactor * nLeft,  1.0f - nLeftFactor  * nBottom, 1.0f - nBottomFactor * nLeftFactor ), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);

						v3DScreen[6 * x + (y * 6 * 16) + 3] = D3DLVERTEX2(D3DVECTOR(nTopFactor    * nRight, 1.0f - nRightFactor * nTop,    1.0f - nTopFactor    * nRightFactor), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * x + (y * 6 * 16) + 4] = D3DLVERTEX2(D3DVECTOR(nBottomFactor * nLeft,  1.0f - nLeftFactor  * nBottom, 1.0f - nBottomFactor * nLeftFactor ), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
						v3DScreen[6 * x + (y * 6 * 16) + 5] = D3DLVERTEX2(D3DVECTOR(nBottomFactor * nRight, 1.0f - nRightFactor * nBottom, 1.0f - nBottomFactor * nRightFactor), D3DCOLOR(0xFFFFFFFF), D3DCOLOR(0), 0, 0, 0, 0);
					}					
				}
			}
			
			// Texture mapping coordinates for the emulator image
			for (int y = 0; y < 16; y++) {
				float tuLeft, tuRight;
				float tvTop, tvBottom;

				tvTop = (float(nDtosHeight) / nTextureHeight) * y / 16;
				tvBottom = (float(nDtosHeight) / nTextureHeight) * (y + 1) / 16;

				for (int x = 0; x < 16; x++) {

					tuLeft = (float(nDtosWidth) / nTextureWidth) * x / 16;
					tuRight = (float(nDtosWidth) / nTextureWidth) * (x + 1) / 16;

					if (bRotateGame) {
						int i = 6 * (15 - x) + ((15 - y) * 6 * 16);
						v3DScreen[i + 0].tu = tuRight; v3DScreen[i + 0].tv = tvTop;
						v3DScreen[i + 1].tu = tuRight; v3DScreen[i + 1].tv = tvBottom;
						v3DScreen[i + 2].tu = tuLeft;  v3DScreen[i + 2].tv = tvTop;
						v3DScreen[i + 3].tu = tuRight; v3DScreen[i + 3].tv = tvBottom;
						v3DScreen[i + 4].tu = tuLeft;  v3DScreen[i + 4].tv = tvTop;
						v3DScreen[i + 5].tu = tuLeft;  v3DScreen[i + 5].tv = tvBottom;
					} else {
						int i = 6 * x + (y * 6 * 16);
						v3DScreen[i + 0].tu = tuLeft;  v3DScreen[i + 0].tv = tvTop;
						v3DScreen[i + 1].tu = tuRight; v3DScreen[i + 1].tv = tvTop;
						v3DScreen[i + 2].tu = tuLeft;  v3DScreen[i + 2].tv = tvBottom;
						v3DScreen[i + 3].tu = tuRight; v3DScreen[i + 3].tv = tvTop;
						v3DScreen[i + 4].tu = tuLeft;  v3DScreen[i + 4].tv = tvBottom;
						v3DScreen[i + 5].tu = tuRight; v3DScreen[i + 5].tv = tvBottom;
					}
				}
			}
		
			// Texture mapping coordinates for the scanlines
			for (int y = 0; y < 16; y++) {
				float tuLeft, tuRight;
				float tvTop, tvBottom;

				tvTop = float(nDtosHeight) * y / 16;
				tvBottom = float(nDtosHeight) * (y + 1) / 16;

				for (int x = 0; x < 16; x++) {

					tuLeft = float(nDtosWidth) * x / 16;
					tuRight = float(nDtosWidth) * (x + 1) / 16;

					if ((!bRotateGame && bRotateEffects) || (bRotateGame && !bRotateEffects)) {
						int i = 6 * (15 - x) + ((15 - y) * 6 * 16);
						v3DScreen[i + 0].tu1 = tuRight; v3DScreen[i + 0].tv1 = tvTop;
						v3DScreen[i + 1].tu1 = tuRight; v3DScreen[i + 1].tv1 = tvBottom;
						v3DScreen[i + 2].tu1 = tuLeft;  v3DScreen[i + 2].tv1 = tvTop;
						v3DScreen[i + 3].tu1 = tuRight; v3DScreen[i + 3].tv1 = tvBottom;
						v3DScreen[i + 4].tu1 = tuLeft;  v3DScreen[i + 4].tv1 = tvTop;
						v3DScreen[i + 5].tu1 = tuLeft;  v3DScreen[i + 5].tv1 = tvBottom;
					} else {
						int i = 6 * x + (y * 6 * 16);
						v3DScreen[i + 0].tu1 = tuLeft;  v3DScreen[i + 0].tv1 = tvTop;
						v3DScreen[i + 1].tu1 = tuRight; v3DScreen[i + 1].tv1 = tvTop;
						v3DScreen[i + 2].tu1 = tuLeft;  v3DScreen[i + 2].tv1 = tvBottom;
						v3DScreen[i + 3].tu1 = tuRight; v3DScreen[i + 3].tv1 = tvTop;
						v3DScreen[i + 4].tu1 = tuLeft;  v3DScreen[i + 4].tv1 = tvBottom;
						v3DScreen[i + 5].tu1 = tuRight; v3DScreen[i + 5].tv1 = tvBottom;
					}
				}
			}
		}
	}

	return 0;
}

static void PutPixel(unsigned char** pSurface, unsigned int nColour)
{
	switch (nScrnDepth) {
		case 15:
			*((unsigned short*)(*pSurface)) = ((nColour >> 8) & 0x7C00) | ((nColour >> 6) & 0x03E0) | ((nColour >> 3) & 0x001F);
			*pSurface += 2;
			break;
		case 16:
			*((unsigned short*)(*pSurface)) = ((nColour >> 8) & 0xF800) | ((nColour >> 5) & 0x07E0) | ((nColour >> 3) & 0x001F);
			*pSurface += 2;
			break;
		case 24:
			(*pSurface)[0] = (nColour >> 16) & 0xFF;
			(*pSurface)[1] = (nColour >>  8) & 0xFF;
			(*pSurface)[2] = (nColour >>  0) & 0xFF;
			*pSurface += 3;
			break;
		case 32:
			*((unsigned int*)(*pSurface)) = nColour;
			*pSurface += 4;
			break;
	}
}

static int vidClearSurface(IDirectDrawSurface7* Surf, RECT* pRect = NULL)
{
	// Clear a surface to black
	DDBLTFX BltFx;

	if (Surf == NULL) {
		return 1;
	}

	memset(&BltFx, 0, sizeof(BltFx));
	BltFx.dwSize = sizeof(BltFx);

	if (FAILED(Surf->Blt(NULL, NULL, pRect, DDBLT_COLORFILL, &BltFx))) {
		return 1;
	} else {
		return 0;
	}
}

// Create a buffer in normal memory
static int vidAllocBurnBuffer()
{
	int nMemLen = 0;
	
	nDtosMemPitch = nDtosWidth * nDtosBpp;
	nMemLen = nDtosHeight * nDtosMemPitch;
	DtosMem = (unsigned char*)malloc(nMemLen);

	if (DtosMem) {
		memset(DtosMem, 0, nMemLen);
		return 0;
	} else {
		return 1;
	}
}

static int ReleaseSurfaces()
{
	// Release effects surfaces
	RELEASE(pRGBEffectTexture);
	RELEASE(pScanlineTexture[0]);
	RELEASE(pScanlineTexture[1]);

	// Release game-image surfaces
	RELEASE(pEmuImage[0]);
	RELEASE(pEmuImage[1]);
	RELEASE(pEmuImage[2]);
	RELEASE(pEmuImage[3]);
	
	nDtosMemPitch = 0;
	free(DtosMem);
	DtosMem = NULL;

	nDtosBpp = 0; nDtosDepth = 0;
	nDtosWidth = 0; nDtosHeight = 0;

	return 0;
}

static int vidExit()
{
	VidSExitOSD();

	free(v3DScreen);
	v3DScreen = NULL;

	ReleaseSurfaces();
	
	VidSRestoreGamma();

	RELEASE(pD3DDevice);
	RELEASE(pD3D);

	RestoreScreenMode();

	if (bUsePageflip == false) {
		RELEASE(pBackbuffer);				// explicitly free the off-screen surface when not usign pageflipping
	}
	RELEASE(pPrimarySurf);
	pBackbuffer = NULL;

	RELEASE(pDD);
	return 0;
}

static HRESULT CALLBACK myEnumTexturessCallback(DDPIXELFORMAT* pddpf, void* pPreferredddpf)
{
    if (pddpf->dwFlags == DDPF_RGB && pddpf->dwRGBBitCount == 16) {

		// Use 15-bit format if supported
		if(pddpf->dwRBitMask == 0x7C00 && pddpf->dwGBitMask == 0x03E0 && pddpf->dwBBitMask == 0x001F) {
            memcpy(pPreferredddpf, pddpf, sizeof(DDPIXELFORMAT) );

			return D3DENUMRET_CANCEL;
        }

		// Use 16-bit format otherwise
		if(pddpf->dwRBitMask == 0xF800 && pddpf->dwGBitMask == 0x07E0 && pddpf->dwBBitMask == 0x001F) {
            memcpy(pPreferredddpf, pddpf, sizeof(DDPIXELFORMAT) );
		}
    }
    return D3DENUMRET_OK;
}

static int vidCreateGameSurfaces()
{
	bool bForceTextureFormat;

	DDSURFACEDESC2 ddsd;
	DDPIXELFORMAT ddpf;
	
	// Determine if we should use a texture format different from the screen format
	bForceTextureFormat = false;
	if (nScrnDepth > 16 && (bVidForce16bit || (!bDoGamma || (nVidFullscreen && bVidUseHardwareGamma) || (!nVidFullscreen && bHardwareGammaOnly)))) {
		memset(&ddpf, 0, sizeof(DDPIXELFORMAT));

		pD3DDevice->EnumTextureFormats(&myEnumTexturessCallback, (void*)&ddpf);

		if (ddpf.dwSize) {
			bForceTextureFormat = true;
		}
	}

	// Create a secondary surface to render the game image onto for the feedback effect
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	
	ddsd.ddsCaps.dwCaps = DDSCAPS_TEXTURE | DDSCAPS_3DDEVICE;

	ddsd.dwWidth = nTextureWidth;
	ddsd.dwHeight = nTextureHeight;

	if (nPreScale & 1) {
		ddsd.dwWidth <<= 1;
	}
	if (nPreScale & 2) {
		ddsd.dwHeight <<= 1;
	}

	memset(&ddsd.ddpfPixelFormat, 0, sizeof(DDPIXELFORMAT));

	if (FAILED(pDD->CreateSurface(&ddsd, &pEmuImage[1], NULL))) {
		bRenderToTexture = false;
	} else {

		vidClearSurface(pEmuImage[1]);
		
		IDirectDrawSurface7* pPreviousTarget;
		if (FAILED(pD3DDevice->GetRenderTarget(&pPreviousTarget))) {
			bRenderToTexture = false;		
		} else {
			if (FAILED(pD3DDevice->SetRenderTarget(pEmuImage[1], 0))) {
				 bRenderToTexture = false;
			 } else {
				 bRenderToTexture = true;
			 }
			 pD3DDevice->SetRenderTarget(pPreviousTarget, 0);
			 pPreviousTarget->Release();
		}
	}

	if (!bRenderToTexture) {

#ifdef PRINT_DEBUG_INFO			
		printf("  * Warning: Device can't render to textures. Using fall-back method for pre-scale/feedback.\n");
#endif
			
		RELEASE(pEmuImage[1]);

		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;

		ddsd.ddsCaps.dwCaps = DDSCAPS_TEXTURE;

		ddsd.dwWidth = nTextureWidth;
		ddsd.dwHeight = nTextureHeight;

		if (nPreScale & 1) {
			ddsd.dwWidth <<= 1;
		}
		if (nPreScale & 2) {
			ddsd.dwHeight <<= 1;
		}

		memset(&ddsd.ddpfPixelFormat, 0, sizeof(DDPIXELFORMAT));

		if (FAILED(pDD->CreateSurface(&ddsd, &pEmuImage[1], NULL))) {
#ifdef PRINT_DEBUG_INFO
			printf("  * Error: Couldn't create secondary game surface(s) for pre-scale/feedback.\n");
#endif
			return 1;
		}

		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;

		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_3DDEVICE | DDSCAPS_VIDEOMEMORY;

		ddsd.dwWidth = nTextureWidth;
		ddsd.dwHeight = nTextureHeight;

		if (nPreScale & 1) {
			ddsd.dwWidth <<= 1;
		}
		if (nPreScale & 2) {
			ddsd.dwHeight <<= 1;
		}

		if (FAILED(pDD->CreateSurface(&ddsd, &pEmuImage[3], NULL))) {
#ifdef PRINT_DEBUG_INFO
			printf("  * Error: Couldn't create secondary game surface(s) for pre-scale/feedback.\n");
#endif
			return 1;
		}
	}

	if (nVidTransferMethod == 1) {
		
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;

		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_3DDEVICE | DDSCAPS_VIDEOMEMORY;

		ddsd.dwWidth = nTextureWidth;
		ddsd.dwHeight = nTextureHeight;

		if (bForceTextureFormat) {
			ddsd.dwFlags |= DDSD_PIXELFORMAT;

			memcpy(&ddsd.ddpfPixelFormat, &ddpf, sizeof(DDPIXELFORMAT));
		}

		if (FAILED(pDD->CreateSurface(&ddsd, &pEmuImage[2], NULL))) {
#ifdef PRINT_DEBUG_INFO
			printf("  * Error: Couldn't create secondary game surface for transfer.\n");
#endif
			return 1;
		}
	}
	
	// Create a surface for the texture that will contain the game image
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);

	ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;

	ddsd.ddsCaps.dwCaps = DDSCAPS_TEXTURE;
	if (nVidTransferMethod == 0) {
		ddsd.ddsCaps.dwCaps2 = DDSCAPS2_HINTDYNAMIC | DDSCAPS2_TEXTUREMANAGE;
	}

	ddsd.dwWidth = nTextureWidth;
	ddsd.dwHeight = nTextureHeight;

	if (bForceTextureFormat) {
		ddsd.dwFlags |= DDSD_PIXELFORMAT;

		memcpy(&ddsd.ddpfPixelFormat, &ddpf, sizeof(DDPIXELFORMAT));
	}

	if (FAILED(pDD->CreateSurface(&ddsd, &pEmuImage[0], NULL))) {
#ifdef PRINT_DEBUG_INFO
		printf("  * Error: Couldn't create game texture surface.\n");
#endif
		return 1;
	}

	vidClearSurface(pEmuImage[0]);

	return 0;
}
	
static int InitEffectsSurfaces()
{
	DDSURFACEDESC2 ddsd;
	
	for (int i = 0, nSize = 2; i < 2; i++, nSize += 2) {
		int Scanlines[2][4] = {{0xFFFFFF, 0x000000, -1, -1}, {0x9F9F9F, 0xFFFFFF, 0x9F9F9F, 0x000000}};
		
		if (pScanlineTexture[i]) {
			
			if (FAILED(pScanlineTexture[i]->IsLost())) {
				if (FAILED(pScanlineTexture[i]->Restore())) {
					return 1;
				}
			}
			
			memset(&ddsd, 0, sizeof(ddsd));
			ddsd.dwSize = sizeof(ddsd);
			if (FAILED(pScanlineTexture[i]->Lock(NULL, &ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WRITEONLY | DDLOCK_DISCARDCONTENTS | DDLOCK_WAIT, NULL))) {
				return 1;
			}

			for (int y = 0; y < nSize; y++) {
				unsigned char* pSurface = (unsigned char*)ddsd.lpSurface + y * ddsd.lPitch;
				int nColour = Scanlines[i][y];
				for (int x = 0; x < nSize; x++) {
					PutPixel(&pSurface, nColour);
				}
			}

			pScanlineTexture[i]->Unlock(NULL);
		}
	}

	if (pRGBEffectTexture) {
		char pFilename[MAX_PATH] = "";
		int nPatternXSize, nPatternYSize;

		if (pRGBEffectTexture == NULL) {
			return 1;
		}

		if (FAILED(pRGBEffectTexture->IsLost())) {
			if (FAILED(pRGBEffectTexture->Restore())) {
				return 1;
			}
		}

		strcpy(pFilename, "pattern\\");
		strcat(pFilename, pRGBEffectPatternName);
		sscanf(pRGBEffectPatternName, "%i x %i", &nPatternXSize, &nPatternYSize);

		FILE* fp = fopen(pFilename, "rb");
		if (fp) {
			fseek(fp, 0, SEEK_END);
			int nSize = nPatternXSize * nPatternYSize * 4;

			// Check if the file is the correct size.
			if (ftell(fp) != nSize) {
				char szTemp[256];

				fclose(fp);
				sprintf(szTemp, "RGB pattern \"%s\" has an incorrect size.\nIt should be %i bytes long.", pRGBEffectPatternName, nSize);
				AppError(szTemp, 1);
				return 1;
			}

			unsigned char* RGBPattern = (unsigned char*)malloc(nSize);
			fseek(fp, 0, SEEK_SET);
			fread(RGBPattern, 1, nSize, fp);
			fclose(fp);

			for (int y = 0; y < nPatternYSize; y++) {
				unsigned char* pPattern = &RGBPattern[y * (nPatternXSize << 2)];
				for (int x = 0; x < nPatternXSize; x++) {
					switch (nRGBEffectPatternmode) {
						case 0:
						case 1: {
							if (nRGBEffectPatternmode == 0 || pPattern[(x << 2) + 3]) {
								int n;
								for (int z = 0; z < 3; z++) {
									n = nRGBEffectIntensity * pPattern[(x << 2) + z] / 256;						
									pPattern[(x << 2) + z] = (n > 255) ? 255 : nRGBEffectFloor + (n * (256 - nRGBEffectFloor) / 256);
								}
							}
							break;
						}
						case 2:
						case 3: {
							int nSaturate = 0;
							int nMax = 0;
							int n;

							if (pPattern[(x << 2) + 0] != pPattern[(x << 2) + 1] || pPattern[(x << 2) + 0] != pPattern[(x << 2) + 2] || pPattern[(x << 2) + 1] != pPattern[(x << 2) + 2]) {

								if (pPattern[(x << 2) + 0] >= pPattern[(x << 2) + 1] && pPattern[(x << 2) + 0] >= pPattern[(x << 2) + 2]) {
									nMax = pPattern[(x << 2) + 0];
									nSaturate |= 1;
								}
								if (pPattern[(x << 2) + 1] >= pPattern[(x << 2) + 0] && pPattern[(x << 2) + 1] >= pPattern[(x << 2) + 2]) {
									nMax = pPattern[(x << 2) + 1];
									nSaturate |= 2;
								}
								if (pPattern[(x << 2) + 2] >= pPattern[(x << 2) + 0] && pPattern[(x << 2) + 2] >= pPattern[(x << 2) + 1]) {
									nMax = pPattern[(x << 2) + 2];
									nSaturate |= 4;
								}

								nSaturate ^= 7;
							}

							for (int z = 0; z < 3; z++) {
								if (nRGBEffectPatternmode == 2 || pPattern[(x << 2) + 3]) {
									n = pPattern[(x << 2) + z];
									if (nSaturate & (1 << z)) {
										n += (nMax - n) * nRGBEffectSaturation / 256;
									}
									n = n * nRGBEffectIntensity / 256;
									pPattern[(x << 2) + z] = (n > 255) ? 255 : nRGBEffectFloor + (n * (256 - nRGBEffectFloor) / 256);
								}
							}
							break;
						}
					}
				}
			}

			memset(&ddsd, 0, sizeof(ddsd));
			ddsd.dwSize = sizeof(ddsd);
			if (FAILED(pRGBEffectTexture->Lock(NULL, &ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WRITEONLY | DDLOCK_DISCARDCONTENTS | DDLOCK_WAIT, NULL))) {
				return 1;
			} else {
				if ((!bRotateGame && bRotateEffects) || (bRotateGame && !bRotateEffects)) {
					for (unsigned int y = 0; y < ddsd.dwHeight; y++) {
						unsigned char* pSurface = (unsigned char*)ddsd.lpSurface + ddsd.lPitch * y;
						unsigned char* pPattern = &RGBPattern[(y % nPatternXSize) << 2];
						for (unsigned int x = 0; x < ddsd.dwWidth; x++) {
							int nColour = pPattern[((x % nPatternYSize) * (nPatternXSize << 2)) + 0] << 16;
							nColour |= pPattern[((x % nPatternYSize) * (nPatternXSize << 2)) + 1] << 8;
							nColour |= pPattern[((x % nPatternYSize) * (nPatternXSize << 2)) + 2];

							PutPixel(&pSurface, nColour);
						}
					}
				} else {
					for (unsigned int y = 0; y < ddsd.dwHeight; y++) {
						unsigned char* pSurface = (unsigned char*)ddsd.lpSurface + ddsd.lPitch * y;
						unsigned char* pPattern = &RGBPattern[(y % nPatternYSize) * nPatternXSize << 2];
						for (unsigned int x = 0; x < ddsd.dwWidth; x++) {
							int nColour = pPattern[((x % nPatternXSize) << 2) + 0] << 16;
							nColour |= pPattern[((x % nPatternXSize) << 2) + 1] << 8;
							nColour |= pPattern[((x % nPatternXSize) << 2) + 2];

							PutPixel(&pSurface, nColour);
						}
					}
				}
				pRGBEffectTexture->Unlock(NULL);

				free(RGBPattern);
			}
		} else {
			AppError("Error reading RGB pattern from disk.", 1);
			return 1;
		}
	}

	return 0;
}

// Create extra surfaces for textures and effects
static int vidCreateEffectsSurfaces()
{
	DDSURFACEDESC2 ddsd;

	// Create the scanline textures
	for (int i = 0, nSize = 2; i < 2; i++, nSize += 2) {
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_TEXTURE | DDSCAPS_VIDEOMEMORY;

		ddsd.dwWidth = nSize;
		ddsd.dwHeight = nSize;

		if (FAILED(pDD->CreateSurface(&ddsd, &pScanlineTexture[i], NULL))) {
#ifdef PRINT_DEBUG_INFO
			printf("  * Error: Couldn't create scanline texture.\n");
#endif
			return 1;
		}
	}

	if (bUseRGBEffects) {
		// Create the RGB effect texture
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
//		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;

		if (nVidFullscreen) {
			if (bVidFullStretch) {
				ddsd.dwWidth = nScrnWidth;
				ddsd.dwHeight = nScrnHeight;
			} else {
				if (bVidArcaderes) {
					ddsd.dwWidth = nGameWidth;
					ddsd.dwHeight = nGameHeight;
				} else {
					RECT rect = { 0, 0, nScrnWidth, nScrnHeight };

					VidSScaleImage(&rect, nGameWidth, nGameHeight, bVidScanRotate);

					ddsd.dwWidth = rect.right - rect.left;
					ddsd.dwHeight = rect.bottom - rect.top;
				}
			}
		} else {
			ddsd.dwWidth = nScrnWidth;
			ddsd.dwHeight = nScrnHeight;
		}

		if (FAILED(pDD->CreateSurface(&ddsd, &pRGBEffectTexture, NULL))) {
#ifdef PRINT_DEBUG_INFO
			printf("  * Error: Couldn't create RGB effects surface.\n");
#endif
			return 1;
		}		
	}

	InitEffectsSurfaces();

	return 0;
}

static int vidAllocSurfaces()
{
	if (pDD == NULL || pD3DDevice == NULL || bDrvOkay == 0) {
		return 1;
	}

	if (bRotateGame) {
		nDtosWidth = nGameHeight;
		nDtosHeight = nGameWidth;
	} else {
		nDtosWidth = nGameWidth;
		nDtosHeight = nGameHeight;
	}

	for (nTextureWidth = 256; nTextureWidth < nDtosWidth; nTextureWidth <<= 1) {}
	for (nTextureHeight = 256; nTextureHeight < nDtosHeight; nTextureHeight <<= 1) {}

#if 0
	if (d3dDeviceDesc.dwTextureCaps & D3DPTEXTURECAPS_SQUAREONLY) {
		if (nTextureWidth < nTextureHeight) {
			nTextureWidth = nTextureHeight;
		}
		if (nTextureHeight < nTextureWidth) {
			nTextureHeight = nTextureWidth;
		}
	}
#endif

	if ((int)d3dDeviceDesc.dwMaxTextureWidth < nTextureWidth || (int)d3dDeviceDesc.dwMaxTextureHeight < nTextureHeight) {
		AppError("Required texture size not supported by 3D hardware.", 1);
		return 1;
	}

	// Allocate all DirectX surfaces
	if (vidCreateGameSurfaces()) {
		ReleaseSurfaces();
		return 1;
	}
	if (vidCreateEffectsSurfaces()) {
		ReleaseSurfaces();
		return 1;
	}
	
	nDtosDepth = VidSGetSurfaceDepth(pEmuImage[0]);	// Get colourdepth of game (texture) surface
	nDtosBpp = (nDtosDepth + 7) >> 3;

	nBurnBpp = nDtosBpp;							// Set Burn library Bytes per pixel
	nBurnBitDepth = nDtosDepth;

	vidAllocBurnBuffer();							// Set up memory buffer for Burn library

	SetBurnHighCol(nDtosDepth);
	BurnRecalcPal();

	return 0;
}

static int vidClear()
{
	if (pPrimarySurf == NULL) {
		return 1;
	}

	if (nVidFullscreen) {		
		vidClearSurface(pPrimarySurf);			// Clear front buffer
		if (bUsePageflip) {
			pPrimarySurf->Flip(NULL, DDFLIP_WAIT);
			vidClearSurface(pPrimarySurf);		// Clear 1st page of backbuffer

			if (bUseTriplebuffer) {
				pPrimarySurf->Flip(NULL, DDFLIP_WAIT);
				vidClearSurface(pPrimarySurf);	// Clear 2nd page of backbuffer
			}

			pPrimarySurf->Flip(NULL, DDFLIP_WAIT);
		} else {
			vidClearSurface(pBackbuffer);		// Clear backbuffer
		}	
	} else {
		vidClearSurface(pBackbuffer);			// Clear backbuffer
	}
	
	return 0;
}

static int vidInitBuffers(bool bTriple)
{
	DDSURFACEDESC2 ddsd;
    DDSCAPS2 ddscaps;

	bUsePageflip = false;
	
	if (nVidFullscreen) {
		if (bTriple) {
			bUsePageflip = true;
		} else {
			DDCAPS ddcaps;

			memset(&ddcaps, 0, sizeof(ddcaps));
			ddcaps.dwSize = sizeof(ddcaps);

			pDD->GetCaps(&ddcaps, NULL);
			if (ddcaps.dwCaps2 & DDCAPS2_FLIPNOVSYNC) {
				bUsePageflip = true;
			}
		}
	}

	// Create the primary surface
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	if (nVidFullscreen && bUsePageflip) {
		// Allocate a complex surface that supports page flipping.
		ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX | DDSCAPS_3DDEVICE | DDSCAPS_VIDEOMEMORY;
		if (bTriple) {
			ddsd.dwBackBufferCount = 2;
		} else {
			ddsd.dwBackBufferCount = 1;
		}
	} else {
		ddsd.dwFlags = DDSD_CAPS;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_VIDEOMEMORY;
	}

	if (FAILED(pDD->CreateSurface(&ddsd, &pPrimarySurf, NULL))) {
		pPrimarySurf = NULL;
		return 1;
	}

	// Get the back buffer
	if (nVidFullscreen && bUsePageflip) {
		memset(&ddscaps, 0, sizeof(ddscaps));
		ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
		if (FAILED(pPrimarySurf->GetAttachedSurface(&ddscaps, &pBackbuffer))) {
			RELEASE(pPrimarySurf);
			pBackbuffer = NULL;
			return 1;
		}
		
		bUseTriplebuffer = bTriple;

	} else {
		// Running in windowed mode or using blt() transfer the final image. Allocate an off-screen surface to render to.
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_3DDEVICE | DDSCAPS_VIDEOMEMORY;
		ddsd.dwWidth = nScrnWidth;
		ddsd.dwHeight = nScrnHeight;
		if (FAILED(pDD->CreateSurface(&ddsd, &pBackbuffer, NULL))) {
			RELEASE(pPrimarySurf);
			pBackbuffer = NULL;
			return 1;
		}
	}

	nScrnDepth = VidSGetSurfaceDepth(pPrimarySurf);	// Get colourdepth of primary surface

	vidClear();										// Clear the surfaces we've just allocated

	return 0;
}

static int vidClipInit()
{
	IDirectDrawClipper *pddcClipper = NULL;			// The clipper

	if (SUCCEEDED(DirectDrawCreateClipper(0, &pddcClipper, NULL))) {
		if (pddcClipper->SetHWnd(0, hVidWnd) >= 0) {
			pPrimarySurf->SetClipper(pddcClipper);
			RELEASE(pddcClipper);
		}
	}
	return 0;
}

static int vidInit()
{
	bUseTriplebuffer = false;

	hVidWnd = hScrnWnd;								// Use Screen window for video

	// Get pointer to DirectDraw device
	DirectDrawCreateEx(NULL, (void**)&pDD, IID_IDirectDraw7, NULL);

	// Get pointer to D3D device
	if (FAILED(pDD->QueryInterface(IID_IDirect3D7, (void**)&pD3D))) {
#ifdef PRINT_DEBUG_INFO
	   	printf("  * Error: Couldn't access Direct3D.\n");
#endif
		vidExit();
		return 1;
	}

	pPrimarySurf = NULL;							// No primary surface yet
	pBackbuffer = NULL;

	// Get the game screen size
	BurnDrvGetScreen(&nGameWidth, &nGameHeight);

	if (BurnDrvGetFlags() & BDF_ROTATE_GRAPHICS_CCW) {
		bRotateGame = true;
	} else {
		bRotateGame = false;
	}
		
	// Set up the display mode
	if (nVidFullscreen) {
		if (EnterFullscreenMode(pDD, nScreenSize, 0, &nScrnWidth, &nScrnHeight)) {
			vidExit();
			return 1;
		}
	} else {
		RECT rect;

		SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0);
		nScrnWidth = rect.right - rect.left;
		nScrnHeight = rect.bottom - rect.top;
		
		pDD->SetCooperativeLevel(hVidWnd, DDSCL_NORMAL);
	}

#ifdef PRINT_DEBUG_INFO
	{
		// Display amount of free video memory
		DDSCAPS2 ddsCaps2;
		DWORD dwTotal;
		DWORD dwFree;

		memset(&ddsCaps2, 0, sizeof(ddsCaps2));
		ddsCaps2.dwCaps = DDSCAPS_PRIMARYSURFACE;

		if (!FAILED(pDD->GetAvailableVidMem(&ddsCaps2, &dwTotal, &dwFree))) {
			printf("  * Initialising video: Total video memory minus display surface(s): %.2fMB.\n", (double)dwTotal / (1024 * 1024));
		}
	}
#endif

	if (bVidTripleBuffer) {
		if (vidInitBuffers(1)) {				// Try to make triple buffer
			printf("  * Warning: Couldn't allocate a triple-buffering surface.\n");
		}
	}
	
	if (pPrimarySurf == NULL) {					// No primary surface yet, so try normal
		vidInitBuffers(0);
	}

	if (pPrimarySurf == NULL) {					// No primary surface, fail
#ifdef PRINT_DEBUG_INFO
	   	printf("  * Error: Couldn't create primary surface.\n");
#endif
		vidExit();
		return 1;
	}

	if (!nVidFullscreen) {						// Make a clipper in windowed mode
		vidClipInit();
	}

	VidSSetupGamma(pPrimarySurf);

	if (bVidScanRotate && nGameWidth < nGameHeight) {
		bRotateEffects = true;
	} else {
		bRotateEffects = false;
	}

	if (bRotateGame) {
		bRotateEffects = !bRotateEffects;
	}

	bUse3DProjection = false;
	if (nVidBlitterOpt[nVidSelect] & 0x00100000) {
		bUse3DProjection = true;
	}
	
	nPreScale = 0;
	if (nVidBlitterOpt[nVidSelect] & 0x01000000) {
		nPreScale = 3;
	}

	bUseRGBEffects = false;
	if (nVidBlitterOpt[nVidSelect] & 0x00010000) {
		int nPreset = -1;

		if (nVidBlitterOpt[nVidSelect] & 0x00020000) {
			RECT rect = { 0, 0, 0, 0 };
			int nScaledSize, nOriginalSize;
			
			GetClientScreenRect(hVidWnd, &rect);
			if (!nVidFullscreen) {
				rect.top += nMenuHeight;
			}
			if (!bVidFullStretch) {
				VidSScaleImage(&rect, nGameWidth, nGameHeight, bVidScanRotate);
			}
			
			if (bVidScanRotate && nGameWidth < nGameHeight) {
				nScaledSize = rect.right - rect.left;
				nOriginalSize = nGameWidth;
			} else {
				nScaledSize = rect.bottom - rect.top;
				nOriginalSize = nGameHeight;
			}

			for (int n = 7; n >= 0; n--) {
				if (nScaledSize >= (n + 1) * nOriginalSize) {
					nPreset = n;
					break;
				}
			}
		} else {
			nPreset = nVidBlitterOpt[nVidSelect] & 0xFF;
		}

		if (nPreset != -1) {
			bVidBilinear = PresetInfo[nPreset].bUseFilter;

			bUseRGBEffects = PresetInfo[nPreset].bUseEffects;
			if (PresetInfo[nPreset].bUseEffects) {
				nRGBEffectIntensity = PresetInfo[nPreset].nIntensity;
				nRGBEffectFloor = PresetInfo[nPreset].nFloor;
				nRGBEffectSaturation = PresetInfo[nPreset].nSaturation;
				nRGBEffectPatternmode = PresetInfo[nPreset].nPatternMode;
				pRGBEffectPatternName = PresetInfo[nPreset].pPatternName;
				nRGBEffectAttenuation = PresetInfo[nPreset].nAttenuation;
				nRGBEffectBlendmode = PresetInfo[nPreset].nBlendMode;
			}
			
			bVidScanlines = PresetInfo[nPreset].bUseScanlines;
			if (PresetInfo[nPreset].bUseScanlines) {
				nRGBScanlineIntensity = PresetInfo[nPreset].nScanIntensity;
			}

//			bVidScanDelay = PresetInfo[nPreset].bUseFeedback;
			if (PresetInfo[nPreset].bUseFeedback) {
				bVidScanDelay = PresetInfo[nPreset].bUseFeedback;
				nVidFeedbackIntensity = PresetInfo[nPreset].nAmount;
				nVidFeedbackOverSaturation = PresetInfo[nPreset].nOverSaturation;
			}
			
            if (PresetInfo[nPreset].bUsePrescale) {
				nVidBlitterOpt[nVidSelect] |= 0x01000000;
			} else {
//				nVidBlitterOpt[nVidSelect] &= ~0x01000000;
			}

			if (nVidBlitterOpt[nVidSelect] & 0x00020000) {
				// Modify the effect a bit if it's rotated
				if ((!bRotateGame && bRotateEffects) || (bRotateGame && !bRotateEffects)) {
					nRGBEffectIntensity = nRGBEffectIntensity * 0xF0 / 0x0100;
					nRGBEffectSaturation += (0x0100 - nRGBEffectSaturation) * 0x70 / 0x0100;
					int nScan = (((nRGBScanlineIntensity >> 16) & 0xFF) * 0xE0 / 0x0100) << 16;
					nScan |= (((nRGBScanlineIntensity >> 8) & 0xFF) * 0xE0 / 0x0100) << 8;
					nScan |= (((nRGBScanlineIntensity >> 0) & 0xFF) * 0xE0 / 0x0100) << 0;
					nRGBScanlineIntensity = nScan;
				}
			}
		}
	}

	if (bVidScanlines) {
		nPreScale &= 2;
	}

	if (bRotateEffects) {
		nPreScale = ((nPreScale >> 1) | (nPreScale << 1)) & 3;
	}

	pD3DDevice = NULL;
	
#ifdef USE_D3D_REFERENCE_DEVICE
	if (FAILED(pD3D->CreateDevice(IID_IDirect3DRefDevice, pBackbuffer, &pD3DDevice))) {
 #ifdef PRINT_DEBUG_INFO
		printf("  * Error: Couldn't access Direct3D Reference device.\n");
 #endif
		vidExit();
		return 1;
	} else {
		printf("  * Using Direct3D Reference device.\n");
	}
#else
	if (bUse3DProjection) {
		if (FAILED(pD3D->CreateDevice(IID_IDirect3DTnLHalDevice, pBackbuffer, &pD3DDevice))) {
			pD3DDevice = NULL;
 #ifdef PRINT_DEBUG_INFO
			printf("  * Warning: Couldn't get TnL 3D hardware, falling back to software TnL.\n");
 #endif
		}
	}
	if (pD3DDevice == NULL) {
		if (FAILED(pD3D->CreateDevice(IID_IDirect3DHALDevice, pBackbuffer, &pD3DDevice))) {
 #ifdef PRINT_DEBUG_INFO
			printf("  * Error: Couldn't access 3D hardware.\n");
 #endif
			vidExit();
			return 1;
		}
	}
#endif
	
	pD3DDevice->GetCaps(&d3dDeviceDesc);									// Get the capabilities of the 3D hardware

	bMultiTexturing = false;
	if (d3dDeviceDesc.wMaxSimultaneousTextures >= 2 && d3dDeviceDesc.wMaxTextureBlendStages >= 2) {
		if (SUCCEEDED(pD3DDevice->SetRenderState(D3DRENDERSTATE_TEXTUREFACTOR, 0x01234567))) {
			bMultiTexturing = true;
		}
	}

#ifdef PRINT_DEBUG_INFO
	if (!bMultiTexturing) {
		printf("  * Warning: Using fall-back method for rendering scanlines.\n");
	}
#endif

	nScanlineOp = D3DTOP_ADD;
	if (d3dDeviceDesc.dwTextureOpCaps & D3DTEXOPCAPS_ADDSMOOTH) {
		nScanlineOp = D3DTOP_ADDSMOOTH;
	}

#ifdef PRINT_DEBUG_INFO
	if (nScanlineOp != D3DTOP_ADDSMOOTH) {
		printf("  * Warning: Using fall-back method for blending scanlines.\n");
	}
#endif

	// Init the buffer surfaces
	if (vidAllocSurfaces()) {
		ReleaseSurfaces();
		return 1;
	}
	
	// Add a viewport
	Viewport.dwX = 0;
	Viewport.dwY = 0;
	Viewport.dwWidth = nScrnWidth;
	Viewport.dwHeight = nScrnHeight;
	
	if (FAILED(pD3DDevice->SetViewport(&Viewport))) {
#ifdef PRINT_DEBUG_INFO
	   	printf("  * Error: Couldn't set initial viewport.\n");
#endif
		vidExit();
		return 1;
	}

	pD3DDevice->SetRenderState(D3DRENDERSTATE_LIGHTING, FALSE);
	pD3DDevice->SetRenderState(D3DRENDERSTATE_CULLMODE, D3DCULL_NONE);
	pD3DDevice->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, TRUE);

	// The polygons for the emulator image on the screen
	if (bRotateGame) {
		vScreen[0] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, float(nDtosWidth) / (nTextureWidth), 0, 0, 0);
		vScreen[1] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, float(nDtosWidth) / (nTextureWidth), float(nDtosHeight) / (nTextureHeight), 0, 0);
		vScreen[2] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, 0, 0, 0, 0);
		vScreen[3] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, 0, float(nDtosHeight) / (nTextureHeight), 0, 0);
	} else {
		vScreen[0] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, 0, 0, 0, 0);
		vScreen[1] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, float(nDtosWidth) / (nTextureWidth), 0, 0, 0);
		vScreen[2] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, 0, float(nDtosHeight) / (nTextureHeight), 0, 0);
		vScreen[3] = D3DTLVERTEX2(D3DVECTOR(0, 0, 0), 1, 0xFFFFFFFF, 0, float(nDtosWidth) / (nTextureWidth), float(nDtosHeight) / (nTextureHeight), 0, 0);
	}

	// Set the texture coordinates for the scanlines
	if ((!bRotateGame && bRotateEffects) || (bRotateGame && !bRotateEffects)) {
		vScreen[0].tu1 = float(nGameHeight); vScreen[0].tv1 = 0;
		vScreen[1].tu1 = float(nGameHeight); vScreen[1].tv1 = float(nGameWidth);
		vScreen[2].tu1 = 0; vScreen[2].tv1 = 0;
		vScreen[3].tu1 = 0; vScreen[3].tv1 = float(nGameWidth);
	} else {
		vScreen[0].tu1 = 0; vScreen[0].tv1 = 0;
		vScreen[1].tu1 = float(nGameWidth); vScreen[1].tv1 = 0;
		vScreen[2].tu1 = 0; vScreen[2].tv1 = float(nGameHeight);
		vScreen[3].tu1 = float(nGameWidth); vScreen[3].tv1 = float(nGameHeight);
	}

	{
		int nWidth = nDtosWidth;
		int nHeight = nDtosHeight;

		if (nPreScale & 1) {
			nWidth <<= 1;
		}
		if (nPreScale & 2) {
			nHeight <<= 1;
		}
		vFeedbackImage[0] = D3DTLVERTEX(D3DVECTOR(-0.5, -0.5, 2.0f), 1, 0, 0, 0, 0);
		vFeedbackImage[1] = D3DTLVERTEX(D3DVECTOR(-0.5 + nWidth, -0.5, 2.0f), 1, 0, 0, float(nDtosWidth) / nTextureWidth, 0);
		vFeedbackImage[2] = D3DTLVERTEX(D3DVECTOR(-0.5, -0.5 + nHeight, 2.0f), 1, 0, 0, 0, float(nDtosHeight) / nTextureHeight);
		vFeedbackImage[3] = D3DTLVERTEX(D3DVECTOR(-0.5 + nWidth, -0.5 + nHeight, 2.0f), 1, 0, 0, float(nDtosWidth) / nTextureWidth, float(nDtosHeight) / nTextureHeight);
	}

	// Force an update of the matrices
	fPreviousScreenAngle = 9999.0f;
	fPreviousScreenCurvature = 9999.0f;

	if (Update3DScreen()) {
#ifdef PRINT_DEBUG_INFO
	   	printf("  * Error: Couldn't initialise 3D geometry.\n");
#endif
		vidExit();
		return 1;
	}

	int nFlags = 0;
	if (nScrnWidth < 1280) {
		nFlags++;
		if (nScrnWidth < 1024) {
			nFlags++;
			if (nScrnWidth < 640) {
				nFlags++;
			}
		}
	}
	VidSInitOSD(pDD, nFlags);

#ifdef PRINT_DEBUG_INFO
	{
		DDSCAPS2 ddsCaps2;
		DWORD dwTotal;
		DWORD dwFree;

		memset(&ddsCaps2, 0, sizeof(ddsCaps2));
		ddsCaps2.dwCaps = DDSCAPS_PRIMARYSURFACE;

		if (!FAILED(pDD->GetAvailableVidMem(&ddsCaps2, &dwTotal, &dwFree))) {
			printf("  * Initialisation complete: %.2fMB video memory free.\n", (double)dwFree / (1024 * 1024));
			printf("    Displaying and rendering in %i-bit mode, emulation running in %i-bit mode.\n", nScrnDepth, nDtosDepth);
			if (nVidFullscreen) {
				printf("    Running in fullscreen mode (%i x %i), ", nScrnWidth, nScrnHeight);
				if (bUseTriplebuffer) {
					printf("using a triple buffer.\n");
				} else {
					if (bUsePageflip) {
						printf("using a double buffer.\n");
					} else {
						printf("using BltFast() to transfer the final image.\n");
					}
				}
			} else {
				printf("    Running in windowed mode, using Blt() to transfer the final image.\n");
			}
		}
	}
#endif

	bLostControl = false;

	return 0;
}

// Render the polygons with the emulator image
static inline void RenderPolygons()
{
	if (bUse3DProjection) {
		if (fVidScreenCurvature == 0.0f) {
			pD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_LVERTEX2, (void*)v3DScreen, 4, 0);
		} else {
			pD3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_LVERTEX2, (void*)v3DScreen, 6 * 16 * 16, 0);
		}
	} else {
		pD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_TLVERTEX2, (void*)vScreen, 4, 0);
	}
}

static int vidRenderImage()
{
	int nPage;

	if (bLostControl) {						// We've lost control of the video hardware
		return 1;
	}
	
	if (FAILED(pBackbuffer->IsLost())) {	// We've lost control of the video hardware
		return 1;
	}

	GetClientScreenRect(hVidWnd, &Dest);
	
	if (nVidFullscreen == 0) {
		Dest.top += nMenuHeight;
		
		if (Dest.left < SystemWorkArea.left || Dest.right > SystemWorkArea.right || Dest.top < SystemWorkArea.top || Dest.bottom > SystemWorkArea.bottom) {
			return 1;
		}
	}

	if (bVidArcaderes && nVidFullscreen) {
		Dest.left = (Dest.right + Dest.left) / 2;
		Dest.left -= nGameWidth / 2;
		Dest.right = Dest.left + nGameWidth;

		Dest.top = (Dest.top + Dest.bottom) / 2;
		Dest.top -= nGameHeight / 2;
		Dest.bottom = Dest.top + nGameHeight;
	} else {
		if (!bVidFullStretch) {
			VidSScaleImage(&Dest, nGameWidth, nGameHeight, bVidScanRotate);
		}
	}
	
    // Paint the screen image

	int nZoom, nScanlineSize = 0;
	if ((bRotateGame && !bRotateEffects) || (!bRotateGame && bRotateEffects)) {
		nZoom = (Dest.right - Dest.left) / nGameWidth;
	} else {
		nZoom = (Dest.bottom - Dest.top) / nGameHeight;
	}

	if (bUseRGBEffects) {
		RECT RGBDest = {0, 0, Dest.right - Dest.left, Dest.bottom - Dest.top};
		pBackbuffer->BltFast(Dest.left, Dest.top, pRGBEffectTexture, &RGBDest, DDBLTFAST_WAIT);
	} else {
		if (bUse3DProjection) {
			RECT rect = { Dest.left, Dest.top, Dest.right, Dest.top + 52 };
			vidClearSurface(pBackbuffer, &rect);
		}
	}

	unsigned int nColour;

	if ((bVidScanDelay || nPreScale)) {
		
		if (FAILED(pD3DDevice->BeginScene())) {
			return 1;
		}
		
		if (bRenderToTexture) {
			pD3DDevice->SetRenderTarget(pEmuImage[1], 0);
		} else {
			pD3DDevice->SetRenderTarget(pEmuImage[3], 0);
		}
				
		Viewport.dwX = 0;
		Viewport.dwY = 0;
		Viewport.dwWidth = nDtosWidth;
		Viewport.dwHeight = nDtosHeight;

		if (nPreScale & 1) {
			Viewport.dwWidth <<= 1;
		}

		if (nPreScale & 2) {
			Viewport.dwHeight <<= 1;
		}

		pD3DDevice->SetViewport(&Viewport);
		
		nColour = (0xFF - nVidFeedbackIntensity) + nVidFeedbackOverSaturation;
		nColour = (nColour > 0x0FF) ? 0xFF : nColour;
		nColour = (nVidFeedbackIntensity << 24) | (nColour << 16) | (nColour << 8) | nColour;
		
		vFeedbackImage[0].color = nColour;
		vFeedbackImage[1].color = nColour;
		vFeedbackImage[2].color = nColour;
		vFeedbackImage[3].color = nColour;
		
		pD3DDevice->SetTexture(0, pEmuImage[0]);
		
		pD3DDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
		pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_POINT);
		pD3DDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
		pD3DDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
		
		if (bVidScanDelay) {
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_CURRENT);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
			
			pD3DDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE);
			pD3DDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_SRCALPHA);
		} else {
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			
			pD3DDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE);
			pD3DDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_ZERO);
		}

		pD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_TLVERTEX, (void*)vFeedbackImage, 4, 0);
		
		if (FAILED(pD3DDevice->EndScene())) {
			return 1;
		}
		
		pD3DDevice->SetTexture(0, NULL);
		
		pD3DDevice->SetRenderTarget(pBackbuffer, 0);

		if (!bRenderToTexture) {
			RECT rect = {0, 0, nDtosWidth, nDtosHeight};

			if (nPreScale & 1) {
				rect.right <<= 1;
			}
			if (nPreScale & 2) {
				rect.bottom <<= 1;
			}

			pEmuImage[1]->BltFast(0, 0, pEmuImage[3], &rect, DDBLTFAST_WAIT);
		}

		nPage = 1;
	} else {
		nPage = 0;
	}
		
	// Set up for the emulator image
	if (FAILED(pD3DDevice->BeginScene())) {
		return 1;
	}
	
	if (bUse3DProjection) {
		
		// It should only do this when the window size/pos has changed
		if (nVidFullscreen == 0) {
			static RECT prevDest = { 0, 0, 0, 0 };
			if (Dest.left != prevDest.left || Dest.top != prevDest.top || Dest.right != prevDest.right || Dest.bottom != prevDest.bottom) {
				prevDest.left = Dest.left;
				prevDest.top = Dest.top;
				prevDest.right = Dest.right;
				prevDest.bottom = Dest.bottom;

				vidClearSurface(pBackbuffer, &Dest);
			}
		}
		
		if (fVidScreenAngle != fPreviousScreenAngle || fVidScreenCurvature != fPreviousScreenCurvature) {
			Update3DScreen();
			vidClearSurface(pBackbuffer);
			SetWorldMatrix();
			SetViewMatrix(2.0f);
			SetProjectionMatrix(2.0f);

			fPreviousScreenAngle = fVidScreenAngle;
			fPreviousScreenCurvature = fVidScreenCurvature;
		}

		Viewport.dwX = Dest.left;
		Viewport.dwY = Dest.top;
		Viewport.dwWidth = Dest.right - Dest.left;
		Viewport.dwHeight = Dest.bottom - Dest.top;
	} else {
		Viewport.dwX = 0;
		Viewport.dwY = 0;
		Viewport.dwWidth = nScrnWidth;
		Viewport.dwHeight = nScrnHeight;
		
		vScreen[0].sx = -0.5f + (float)Dest.left;
		vScreen[0].sy = -0.5f + (float)Dest.top;
		vScreen[1].sx = -0.5f + (float)Dest.right;
		vScreen[1].sy = -0.5f + (float)Dest.top;
		vScreen[2].sx = -0.5f + (float)Dest.left;
		vScreen[2].sy = -0.5f + (float)Dest.bottom;
		vScreen[3].sx = -0.5f + (float)Dest.right;
		vScreen[3].sy = -0.5f + (float)Dest.bottom;
	}
	
	pD3DDevice->SetViewport(&Viewport);

	if (bUseRGBEffects) {
		nColour = ((0xFF - nRGBEffectAttenuation) << 24) | nRGBScanlineIntensity;
		if (nRGBEffectBlendmode == 0) {
			pD3DDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_SRCALPHA);
			pD3DDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_SRCCOLOR);
		} else {
			pD3DDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_DESTCOLOR);
			pD3DDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_ZERO);
		}		
	} else {
		nColour = ((0xFF - nRGBEffectAttenuation) << 24) | nVidScanIntensity;
		pD3DDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE);
		pD3DDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_ZERO);
	}

	int bScanlines = 0;
	if (nZoom >= 2 && bVidScanlines) {
		bScanlines = 1;
		if (nZoom >= 4) {
			nScanlineSize++;
		}
	}
		
	if (bMultiTexturing) {
		
		pD3DDevice->SetRenderState(D3DRENDERSTATE_TEXTUREFACTOR, nColour);

		if (bScanlines) {
			if (bVidScanBilinear) {
				pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_LINEAR);
				pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_LINEAR);
			} else {
				pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_POINT);
				pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_POINT);
			}
			if (bVidBilinear) {
				pD3DDevice->SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTFG_LINEAR);
				pD3DDevice->SetTextureStageState(1, D3DTSS_MINFILTER, D3DTFG_LINEAR);
			} else {
				pD3DDevice->SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTFG_POINT);
				pD3DDevice->SetTextureStageState(1, D3DTSS_MINFILTER, D3DTFG_POINT);
			}

			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TFACTOR);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, nScanlineOp);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 1);
			pD3DDevice->SetTextureStageState(1, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			pD3DDevice->SetTextureStageState(1, D3DTSS_COLORARG2, D3DTA_CURRENT);
			pD3DDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_MODULATE);
			pD3DDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			pD3DDevice->SetTextureStageState(1, D3DTSS_TEXCOORDINDEX, 0);

			pD3DDevice->SetTexture(0, pScanlineTexture[nScanlineSize]);
			pD3DDevice->SetTexture(1, pEmuImage[nPage]);

		} else {
			if (bVidBilinear) {
				pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_LINEAR);
				pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_LINEAR);
			} else {
				pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_POINT);
				pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_POINT);
			}

			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TFACTOR);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
			pD3DDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);

			pD3DDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
			pD3DDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

			pD3DDevice->SetTexture(0, pEmuImage[nPage]);
		}

		RenderPolygons();

		pD3DDevice->SetTexture(0, NULL);
		pD3DDevice->SetTexture(1, NULL);
	
	} else {						// multi-texturing not supported by hardware
		
		if (bUse3DProjection) {
			static unsigned int nPrevColour = 0xFFFFFFFF;
			if (nColour != nPrevColour) {
				for (int i = 0; i < 16 * 16 * 6; i++) {
					v3DScreen[i].color = nColour;
				}
				nPrevColour = nColour;
			}
		} else {
			vScreen[0].color = nColour;
			vScreen[1].color = nColour;
			vScreen[2].color = nColour;
			vScreen[3].color = nColour;
		}

		if (bVidBilinear) {
			pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_LINEAR);
			pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_LINEAR);
		} else {
			pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_POINT);
			pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_POINT);
		}

		pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
		pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
		pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
		pD3DDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);

		pD3DDevice->SetTexture(0, pEmuImage[nPage]);

		pD3DDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
		pD3DDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

		RenderPolygons();
	
		pD3DDevice->SetTexture(0, NULL);

		if (bScanlines) {

			if (bVidScanBilinear) {
				pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_LINEAR);
				pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_LINEAR);
			} else {
				pD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFG_POINT);
				pD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFG_POINT);
			}

			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, nScanlineOp);
			pD3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
			pD3DDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 1);

			pD3DDevice->SetTexture(0, pScanlineTexture[nScanlineSize]);
			
			pD3DDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
			pD3DDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

			pD3DDevice->SetRenderState(D3DRENDERSTATE_SRCBLEND, D3DBLEND_DESTCOLOR);
			pD3DDevice->SetRenderState(D3DRENDERSTATE_DESTBLEND, D3DBLEND_ZERO);
			
			RenderPolygons();

			pD3DDevice->SetTexture(0, NULL);
		}
	}

	if (FAILED(pD3DDevice->EndScene())) {
		return 1;
	}

	// Display OSD text message
	VidSDisplayOSD(pBackbuffer, &Dest);

	return 0;
}

// Copy the game image to the texture and render the image using 3D hardware
static int vidBurnToSurf()
{
	DDSURFACEDESC2 ddsd;

	RECT rect = { 0, 0, nDtosWidth, nDtosHeight };

	unsigned char *pd, *ps;
	unsigned char *Surf;
	int nPitch;

	if (bLostControl) {						// We've lost control of the video hardware
		return 1;
	}
	
	// Lock the surface so we can write to it
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	
	if (nVidTransferMethod == 1) {
		// Copy the image to a surface (located in video memory), then use bltfast() to blit it to the texture
		if (FAILED(pEmuImage[2]->Lock(NULL, &ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL))) {
			return 1;
		}

		Surf = (unsigned char*)ddsd.lpSurface;
		nPitch = ddsd.lPitch;

		pd = Surf; ps = DtosMem;
		for (int y = 0; y < nDtosHeight; y++, pd += nPitch, ps += nDtosMemPitch) {
			memcpy(pd, ps, nDtosMemPitch);
		}

		pEmuImage[2]->Unlock(NULL);

		pEmuImage[0]->BltFast(0, 0, pEmuImage[2], &rect, DDBLTFAST_WAIT);	
	} else {
		// Use the surface supplied by DirectX texture management and let it perform the blit
		if (FAILED(pEmuImage[0]->Lock(&rect, &ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WRITEONLY | DDLOCK_DISCARDCONTENTS | DDLOCK_WAIT, NULL))) {
			return 1;
		}

		Surf = (unsigned char*)ddsd.lpSurface;
		nPitch = ddsd.lPitch;

		pd = Surf; ps = DtosMem;
		for (int y = 0; y < nDtosHeight; y++, pd += nPitch, ps += nDtosMemPitch) {
			memcpy(pd, ps, nDtosMemPitch);
		}

		pEmuImage[0]->Unlock(NULL);
	}
	
	return vidRenderImage();
}

// Run one frame and render the screen
static int vidFrame(bool bRedraw)			// bRedraw = 0
{
	// Check if we've lost control of the video hardware
	if (FAILED(pPrimarySurf->IsLost()) || FAILED(pDD->TestCooperativeLevel())) {
		bLostControl = true;
	}
	
	// If we've lost control, see if we've regained it and attempt to restore surfaces
	if (bLostControl) {
		if (FAILED(pDD->TestCooperativeLevel())) {
			return 1;
		}

		if (FAILED(pPrimarySurf->IsLost())) {
			if (FAILED(pPrimarySurf->Restore())) {
				return 1;
			}
		}
		
		if (!bUsePageflip) {
			if (FAILED(pBackbuffer->IsLost())) {
				if (FAILED(pBackbuffer->Restore())) {
					return 1;
				}
			}
		}

		for (int i = 0; i < 4; i++) {
			
			// Skip texture 0 if we're using DirectX texture management
			if (nVidTransferMethod == 0 && i == 0) {
				continue;
			}
			
			if (pEmuImage[i]) {
				if (FAILED(pEmuImage[i]->IsLost())) {
					if (FAILED(pEmuImage[i]->Restore())) {
						return 1;
					}
				}
			}
		}

		if (InitEffectsSurfaces()) {		// Restore RGB effects and/or scanline surfaces
			return 1;
		}

		if (VidSRestoreOSD()) {
			return 1;
		}

		vidClear();							// Clear screen

		bLostControl = false;
	}
	
	pSShotBuffer = NULL;
	pBurnDraw = DtosMem;
	nBurnPitch = nDtosMemPitch;

	if (bDrvOkay) {
		if (bRedraw) {						// Redraw current frame
			if (BurnDrvRedraw()) {
				BurnDrvFrame();				// No redraw function provided, advance one frame
			}
		} else {
			BurnDrvFrame();					// Run one frame and draw the screen
		}
	} else {
		// Blank the screen
		memset(DtosMem, 0, nDtosMemPitch * nDtosHeight);
	}
	pSShotBuffer = pBurnDraw;
	pBurnDraw = NULL;
	nBurnPitch = 0;

	return vidBurnToSurf();					// Copy the memory buffer and use 3D hardware to render the image
}

// Paint the Dtos surface onto the primary surface
int vidPaint(int bValidate)
{
	RECT rect;
	
	if (bLostControl) {						// We've lost control of the video hardware
		return 1;
	}
	
	if (FAILED(pPrimarySurf->IsLost())) {	// We've lost control of the video hardware
		return 1;
	}

	if (!bUsePageflip) {
		GetClientScreenRect(hVidWnd, &rect);
		
		rect.top += nMenuHeight;
		
		if (!bVidFullStretch) {
			VidSScaleImage(&rect, nGameWidth, nGameHeight, bVidScanRotate);
		}

		if ((rect.right - rect.left) != (Dest.right - Dest.left) || (rect.bottom - rect.top ) != (Dest.bottom - Dest.top)) {
			bValidate |= 2;
		}
	}

	if (bValidate & 2) {
		vidRenderImage();
	}

	// Display final image
	if (bUsePageflip) {
		if (bUseTriplebuffer) {
			if (FAILED(pPrimarySurf->Flip(NULL, DDFLIP_WAIT))) {
				return 1;
			}
		} else {
			if (FAILED(pPrimarySurf->Flip(NULL, DDFLIP_NOVSYNC | DDFLIP_WAIT))) {
				return 1;
			}
		}
	} else {
		if (nVidFullscreen) {
			if (FAILED(pPrimarySurf->BltFast(Dest.left, Dest.top, pBackbuffer, &Dest, DDBLTFAST_WAIT))) {
				return 1;
			}
		} else {
			if (FAILED(pPrimarySurf->Blt(&rect, pBackbuffer, &Dest, DDBLT_ASYNC, NULL))) {
				if (FAILED(pPrimarySurf->Blt(&rect, pBackbuffer, &Dest, DDBLT_WAIT, NULL))) {
					return 1;
				}
			}
		}
		
		if (bValidate & 1) {
			// Validate the rectangle we just drew
			POINT c = {0, 0};
			ClientToScreen(hVidWnd, &c);
			rect.left -= c.x; rect.right -= c.x;
			rect.top -= c.y; rect.bottom -= c.y;
			ValidateRect(hVidWnd, &rect);
		}
	}

	return 0;
}

static int vidScale(RECT* pRect, int nWidth, int nHeight)
{
	return VidSScaleImage(pRect, nWidth, nHeight, bVidScanRotate);
}

// The video output plugin:
struct VidOut VidOutD3D = {vidInit, vidExit, vidFrame, vidPaint, vidScale};

