// Video Output - (calls all the Vid Out plugins)
#include "app.h"

extern struct VidOut VidOutDDraw;
extern struct VidOut VidOutD3D;
extern struct VidOut VidOutBlitFX;

static struct VidOut *pVidOut[] = {
	&VidOutDDraw,
	&VidOutD3D,
	&VidOutBlitFX,
};

#define VID_LEN (sizeof(pVidOut) / sizeof(pVidOut[0]))

HWND hVidWnd = NULL;					// The window to be used for video
int nVidWidth = 640, nVidHeight = 480, nVidDepth = 16, nVidRefresh = 0;
int nVidFullscreen = 0;
int bVidFullStretch = 0;				// 1 = stretch to fill the entire window/screen
int bVidCorrectAspect = 1;				// 1 = stretch to fill the window/screen while maintaining the correct aspect ratio
int bVidTripleBuffer = 0;				// 1 = use triple buffering
int bVidBilinear = 1;					// 1 = enable bi-linear filtering (D3D blitter)
int bVidScanlines = 0;					// 1 = draw scanlines
int bVidScanRotate = 1;					// 1 = rotate scanlines and RGB effects for rotated games
int bVidScanBilinear = 1;				// 1 = use bi-linear filtering for scanlines (D3D blitter, debug variable)
int nVidScanIntensity = 0x00BFBFBF;		// The maximum colour-value for the scanlines (D3D blitter)
int bVidScanHalf = 1;					// Draw scanlines at half intensity instead of black (DDraw blitter)
int bVidScanDelay = 0;					// Blend the previous image with the current one
int nVidFeedbackIntensity = 0x40;		// Blend factor for previous frame (D3D blitter)
int nVidFeedbackOverSaturation = 0x00;	// Add this to the current frame blend factor
int bVidUseHardwareGamma = 1;			// Use the video hardware when correcting gamma
int bVidArcaderes = 0;					// Use game resolution for fullscreen modes
int bVidForce16bit = 1;					// Emulate the game in 16-bit even when the screen is 32-bit (D3D blitter)
int nVidTransferMethod = 1;				// How to transfer the game image to a texture --
										// 0 = use driver/DirectX texture management
										// 1 = copy to a video memory surface, then use bltfast()
float fVidScreenAngle = 0.174533f;		// The angle at which to tilt the screen backwards (in radians, D3D blitter)
float fVidScreenCurvature = 0.698132f;	// The angle of the maximum screen curvature (in radians, D3D blitter)

int nVidBlitterOpt[VID_LEN] = {0, };	// Options for the blitter module (meaning depens on module)

unsigned int nVidSelect = 0;			// Which video output is selected
static unsigned int nVidActive = 0;
static bool bVidOkay = false;

int VidSelect(unsigned int nPlugin)
{
	if (nPlugin < VID_LEN) {
		nVidSelect = nPlugin;
		return 0;
	} else {
		return 1;
	}
}

// Forward to VidOut functions
int VidInit()
{
	int nRet = 1;
	VidExit();
	if ((nVidSelect < VID_LEN) &&  bDrvOkay) {
		nVidActive = nVidSelect;
		if ((nRet = pVidOut[nVidActive]->Init()) == 0) {
			bVidOkay = true;
		}
	}
	return nRet;
}

int VidExit()
{
	if (bVidOkay) {
		bVidOkay = false;
		int nRet = pVidOut[nVidActive]->Exit();

		nBurnPitch = nBurnBpp = nBurnBitDepth = 0;

		return nRet;

	} else {
		return 1;
	}
}

int VidFrame()
{
	if (bVidOkay && bDrvOkay) {
		return pVidOut[nVidActive]->Frame(0);
	} else {
		return 1;
	}
}

int VidRedraw()
{
	if (bVidOkay && bDrvOkay) {
		return pVidOut[nVidActive]->Frame(1);
	} else {
		return 1;
	}
}

// If bValidate is 1, the video code should use ValidateRect
// to validate the hVidWnd rectangle it draws.
int VidPaint(int bValidate)
{
	if (bVidOkay) {
		return pVidOut[nVidActive]->Paint(bValidate);
	} else {
		return 1;
	}
}

int VidImageSize(RECT* pRect, int nGameWidth, int nGameHeight)
{
 	if (bVidOkay) {
		return pVidOut[nVidActive]->ImageSize(pRect, nGameWidth, nGameHeight);
	} else {
		return pVidOut[nVidSelect]->ImageSize(pRect, nGameWidth, nGameHeight);
	}
}

