// Software blitter effects via DirectDraw
#include "app.h"
#include "directx_support.h"

#define uint32 unsigned long
#define uint16 unsigned short
#define uint8 unsigned char

#ifdef __GNUC__
 #define USE_ASM_i586
 #include "iscale.h"
#endif

extern "C" void __cdecl _2xSaISuperEagleLine(uint8* srcPtr, uint8* deltaPtr, uint32 srcPitch, uint32 width, uint8* dstPtr, uint32 dstPitch, uint16 dstBlah);
extern "C" void __cdecl _2xSaILine(uint8* srcPtr, uint8* deltaPtr, uint32 srcPitch, uint32 width, uint8* dstPtr, uint32 dstPitch, uint16 dstBlah);
extern "C" void __cdecl _2xSaISuper2xSaILine(uint8* srcPtr, uint8* deltaPtr, uint32 srcPitch, uint32 width, uint8* dstPtr, uint32 dstPitch, uint16 dstBlah);
extern "C" void __cdecl Init_2xSaIMMX(uint32 BitFormat);

static IDirectDraw7* BlitFXDD = NULL;				// DirectDraw interface
static IDirectDrawSurface7* BlitFXPrim = NULL;		// Primary surface
static IDirectDrawSurface7* BlitFXBack = NULL;		// Back buffer surface

static int nBlitFXWidth = 0, nBlitFXHeight = 0;		// screen size
static int nGameWidth = 0, nGameHeight = 0;			// screen size
static int nBlitFXDepth = 0;						// Bit depth
static unsigned char* BlitFXMem = NULL;		   		// Memory buffer
static unsigned char* GameMem = NULL;		   		// Memory buffer
static unsigned char* BlitFXXBuffer = NULL;			// Scratch for the 2xSaI/SuperEagle blitters
static int nBlitFXMemPitch = 0;						// Memory buffer pitch
static int nGameMemPitch = 0;						// Memory buffer pitch
IDirectDrawSurface7* pddsBlitFX[2] = {NULL, };		// The image surfaces

static int nEnlarge;
static int nUseBlitter;

static int nUseSys;
static int nDirectAccess = 1;
static bool bRotateGame = false;

static int BlitFXInit();
static int BlitFXExit();
static int Exit();

static int Clear(IDirectDrawSurface7* Surf)
{
	// Clear a surface to black
	DDBLTFX BltFx;

	if (Surf == NULL) {
		return 1;
	}

	memset(&BltFx, 0, sizeof(BltFx));
	BltFx.dwSize = sizeof(BltFx);

	if (Surf->Blt(NULL, NULL, NULL, DDBLT_COLORFILL, &BltFx) < 1) {
		return 1;
	} else {
		return 0;
	}
}

static int PrimClear()
{
	if (BlitFXPrim == NULL) {
		return 1;
	}
	Clear(BlitFXPrim);								// Clear 1st page

	if (BlitFXBack == NULL) {
		return 0;
	}
	Clear(BlitFXBack);								// Clear 2nd page
	BlitFXPrim->Flip(NULL, DDFLIP_WAIT);			// wait till the flip actually occurs
	Clear(BlitFXBack);								// Clear 3rd page

	return 0;
}

static int PrimInit(int bTriple)
{
	DDSURFACEDESC2 ddsd;
	int nRet = 0;

	// Create the primary surface
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	if (bTriple) {
		// Make a primary surface capable of triple bufferring
		ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX | DDSCAPS_VIDEOMEMORY;
		ddsd.dwBackBufferCount = 2;
	} else {
		ddsd.dwFlags = DDSD_CAPS;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
	}

	BlitFXPrim = NULL;
	nRet = BlitFXDD->CreateSurface(&ddsd, &BlitFXPrim, NULL);
	if (nRet < 0 || BlitFXPrim == NULL) {
		return 1;
	}
	
	if (bTriple == 0) {								// We are done
		return 0;
	}

	// Get the back buffer
	memset(&ddsd.ddsCaps, 0, sizeof(ddsd.ddsCaps));
	ddsd.ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;

	BlitFXBack = NULL;
	nRet = BlitFXPrim->GetAttachedSurface(&ddsd.ddsCaps, &BlitFXBack);
	if (nRet < 0 || BlitFXBack == NULL) {			// Failed to make triple buffer
		RELEASE(BlitFXPrim)
		return 1;
	}

	PrimClear();									// Clear all three pages
	return 0;
}

static int ClipInit()
{
	IDirectDrawClipper *pddcClipper = NULL;		// The clipper
	
	DirectDrawCreateClipper(0, &pddcClipper, NULL);

	if (pddcClipper) {
		if (pddcClipper->SetHWnd(0, hVidWnd) >= 0) {
			BlitFXPrim->SetClipper(pddcClipper);
			RELEASE(pddcClipper);
		}
	}
	return 0;
}

static bool MMXSupport()
{
#if 0
	unsigned int nHasMMX;
	
	__asm__ __volatile__ (
		"mov 1, %%eax\n"
		"cpuid\n"
		"and 0x00800000, %%edx\n"					// bit 23 indicates MMX support
		"mov %%edx, %0\n"
		
		: "=g" (nHasMMX)
		: "g" (nHasMMX)
		: "eax", "ebx", "ecx", "edx", "cc"
	);
	
	if (nHasMMX) {
		return true;
	} else {
		return false;
	}
#else
	return true;
#endif
}

static int Init()
{
	nUseBlitter = nVidBlitterOpt[nVidSelect] & 0xFF;

	if (hScrnWnd == NULL) {
		return 1;
	}
	
	hVidWnd = hScrnWnd;								// Use Screen window for video
		
	// Get pointer to DirectDraw device
	DirectDrawCreateEx(NULL, (void**)&BlitFXDD, IID_IDirectDraw7, NULL);

	if (BurnDrvGetFlags() & BDF_ROTATE_GRAPHICS_CCW) {
		bRotateGame = true;
	} else {
		bRotateGame = false;
	}
	
	BlitFXPrim = NULL;								// No primary surface yet
	BlitFXBack = NULL;

	bVidScanlines = 0;								// !!!
	
	// Remember the changes to the display
	if (nVidFullscreen) {
		int nZoom = 2;
		if ((nVidBlitterOpt[nVidSelect] & 0x0100) == 0) {
			nZoom = nScreenSize;
		}
		
		if (EnterFullscreenMode(BlitFXDD, nZoom, 16, NULL, NULL)) {
			Exit();
			return 1;
		}

		if (bVidTripleBuffer) {
			if (PrimInit(1)) {						// Try to make triple buffer
				AppError("Could not make Triple Buffer", 1);
				// If we fail, fail entirely and make a normal buffer
				RELEASE(BlitFXBack);
				RELEASE(BlitFXPrim);
			}
		}
	} else {
		nEnlarge = 1;
		BlitFXDD->SetCooperativeLevel(hVidWnd, DDSCL_NORMAL);
	}

	if (BlitFXPrim == NULL) {
		// No primary surface yet, so try normal
		if (PrimInit(0)) {
			RELEASE(BlitFXBack);
			RELEASE(BlitFXPrim);
		}
	}

	if (BlitFXPrim == NULL) {						// No primary surface
		Exit();
		return 1;
	}

	// Make a clipper in windowed mode
	if (nVidFullscreen == 0) {
		ClipInit();
	}

	VidSSetupGamma(BlitFXPrim);

	// Initialize the buffer surfaces
	BlitFXInit();

	if (MMXSupport() == false || nEnlarge == 0 || nBlitFXDepth == 24 || (nUseBlitter == 1 && nBlitFXDepth == 15) || ((nUseBlitter >= 2 && nUseBlitter <= 4) && nBlitFXDepth == 32)) {
		nUseBlitter = 0;
	}

	if (nUseBlitter >= 2 && nUseBlitter <= 4) {						// Initialize the 2xSaI engine
		BlitFXXBuffer = (unsigned char*)malloc(nBlitFXHeight * nBlitFXMemPitch);
		if (BlitFXXBuffer == NULL) {
			Exit();
			return 1;
		}		
		
		if (nBlitFXDepth == 15) {
			Init_2xSaIMMX(555);
		} else {
			Init_2xSaIMMX(565);
		}
	}

	VidSInitOSD(BlitFXDD, 0);

	return 0;
}

static int Exit()
{
	VidSExitOSD();
	
	free(BlitFXXBuffer);
	BlitFXXBuffer = NULL;
	
	BlitFXExit();

	VidSRestoreGamma();

	RELEASE(BlitFXPrim);							// a single call releases all surfaces
	BlitFXBack = NULL;

	RestoreScreenMode();

	RELEASE(BlitFXDD)
	
	return 0;
}

// Create a buffer in normal memory
static int BlitFXMakeMem()
{
	int nMemLen = 0;

	nBlitFXMemPitch = nBlitFXWidth * ((nBlitFXDepth + 7) >> 3);
	nMemLen = (nBlitFXHeight + 1) * nBlitFXMemPitch;
	BlitFXMem = (unsigned char*)malloc(nMemLen);
	
	if (bRotateGame) {
		nGameMemPitch = nGameWidth * ((nBlitFXDepth + 7) >> 3);
		GameMem = (unsigned char*)malloc(nMemLen);
	} else {
		nGameMemPitch = nBlitFXMemPitch;
		GameMem = BlitFXMem;
	}
	
	if (BlitFXMem) {
		memset(BlitFXMem, 0, nMemLen);
		return 0;
	} else {
		return 1;
	}
}

static int AutodetectUseSys()
{
	// Try to autodetect the best secondary buffer type to use, based on the cards capabilities
	DDCAPS ddc;
	if (BlitFXDD == NULL) {
		return 1;
	}
	memset(&ddc, 0, sizeof(ddc));
	ddc.dwSize = sizeof(ddc);
	BlitFXDD->GetCaps(&ddc, NULL);

	if (ddc.dwCaps & DDCAPS_BLTSTRETCH) {	// If it can do a hardware stretch use video memory
		return 0;
	} else {								// Otherwise use system memory:
		return 1;
	}
}

// Create a secondary DD surface for the screen
static int BlitFXMakeSurf()
{
	int nRet;
	DDSURFACEDESC2 ddsd;

	if (BlitFXDD == NULL) {
		return 1;
	}

	nUseSys = nUseSysMemory;
	if (nUseSys < 0) {
		nUseSys = AutodetectUseSys();
	}

	nDirectAccess = 0;
	if (nUseSys == 0 && nUseBlitter != 0) {
		nDirectAccess = (nVidBlitterOpt[nVidSelect] >> 9) & 1 ^ 1;
	}

	pddsBlitFX[0] = NULL;
	pddsBlitFX[1] = NULL;

	// Try to allocate buffer in Video memory if required, always allocate one in System memory
	if (nUseSys == 0) {
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;

		if (nEnlarge) {
			ddsd.dwWidth = nGameWidth << 1;
			ddsd.dwHeight = nGameHeight << 1;
		} else {
			ddsd.dwWidth = nGameWidth;
			ddsd.dwHeight = nGameHeight;
		}

		nRet = BlitFXDD->CreateSurface(&ddsd, &pddsBlitFX[0], NULL);
	
		if (nRet < 0 || pddsBlitFX[0] == NULL) {									// Allocation in Video meory has failed, so use System memory only
			pddsBlitFX[0] = NULL;
			nDirectAccess = 0;
			nUseSys = 1;
		}
	}

	if (nDirectAccess == 0) {
		memset(&ddsd, 0, sizeof(ddsd));
		ddsd.dwSize = sizeof(ddsd);
		ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;

		if (nEnlarge) {
			ddsd.dwWidth = nGameWidth << 1;
			ddsd.dwHeight = nGameHeight << 1;
		} else {
			ddsd.dwWidth = nGameWidth;
			ddsd.dwHeight = nGameHeight;
		}

		nRet = BlitFXDD->CreateSurface(&ddsd, &pddsBlitFX[1], NULL);

		if (nRet < 0 || pddsBlitFX[1] == NULL) {
			return 1;
		}
	}
	
	Clear(pddsBlitFX[1 ^ nDirectAccess]);

	return 0;
}

static int BlitFXInit()
{
	if (BlitFXDD == NULL || bDrvOkay == 0) {
		return 1;
	}

	// Get the game screen size
	BurnDrvGetScreen(&nGameWidth, &nGameHeight);
	
	if (bRotateGame) {
		nBlitFXWidth = nGameHeight;
		nBlitFXHeight = nGameWidth;
	} else {
		nBlitFXWidth = nGameWidth;
		nBlitFXHeight = nGameHeight;
	}

	RECT rect = { 0, 0, 0, 0 };
	GetClientScreenRect(hVidWnd, &rect);
	if (!nVidFullscreen) {
		rect.top += nMenuHeight;
	}

	nBlitFXDepth = VidSGetSurfaceDepth(BlitFXPrim);	// Get color depth of primary surface
	nBurnBitDepth = nBlitFXDepth;
	nBurnBpp = (nBlitFXDepth + 7) >> 3;				// Set Burn library Bytes per pixel

	// Use our callback to get colors:
	SetBurnHighCol(nBlitFXDepth);
	// Recalculate the whole palette next time BurnDraw is called
	// This will use our callback for the whole palette
	BurnRecalcPal();

	// Make the normal memory buffer
	if (BlitFXMakeMem()) {
		BlitFXExit();
		return 1;
	}

	// Make the DirectDraw secondary surface
	if (BlitFXMakeSurf()) {
		BlitFXExit();
		return 1;
	}

	return 0;
}

static int BlitFXExit()
{
	RELEASE(pddsBlitFX[0]);
	RELEASE(pddsBlitFX[1]);
	nBlitFXMemPitch = 0;
	free(BlitFXMem);
	if (GameMem != BlitFXMem) {
		free(GameMem);
	}
	BlitFXMem = NULL;
	GameMem = NULL;
	bRotateGame = false;

	nBlitFXWidth = 0;
	nBlitFXHeight = 0;

	return 0;
}

// Copy BlitFXsMem to pddsBlitFX
static int MemToSurf()
{
	DDSURFACEDESC2 ddsd;

	unsigned char* Surf;
	int nPitch;

	if (pddsBlitFX == NULL) {
		return 1;
	}

	if (BlitFXPrim->IsLost()) {						// We've lost control of the screen
		return 1;
	}

	if (bRotateGame) {
		unsigned char* ps;
		unsigned char* pd = GameMem;
		switch ((nBlitFXDepth + 7) >> 3) {
			case 4: {
				for (int y = 0; y < nGameHeight; y++) {
					ps = BlitFXMem + (nGameHeight - 1 - y) * 4;
					for (int x = 0; x < nGameWidth; x++) {
						*(int*)pd = *(int*)ps;
						ps += nBlitFXMemPitch;
						pd += 4;
					}
				}
				break;
			}
			case 3: {
				for (int y = 0; y < nGameHeight; y++) {
					ps = BlitFXMem + (nGameHeight - 1 - y) * 3;
					for (int x = 0; x < nGameWidth; x++) {
						pd[0] = ps[0];
						pd[1] = ps[2];
						pd[2] = ps[2];
						ps += nBlitFXMemPitch;
						pd +=3;
					}
				}
				break;
			}
			case 2:	{
				for (int y = 0; y < nGameHeight; y++) {
					ps = BlitFXMem + (nGameHeight - 1 - y) * 2;
					for (int x = 0; x < nGameWidth; x++) {
						*(short*)pd = *(short*)ps;
						ps += nBlitFXMemPitch;
						pd += 2;
					}
				}
				break;
			}
		}
	}
	
	// Lock the surface so we can write to it
	memset(&ddsd, 0, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	pddsBlitFX[1 ^ nDirectAccess]->Lock(NULL, &ddsd, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL);
	if (ddsd.lpSurface == NULL) {
		return 1;
	}

	Surf = (unsigned char *)ddsd.lpSurface;
	nPitch = ddsd.lPitch;

	// Copy the image
	switch (nUseBlitter) {

#ifndef __GNUC__
		case 1:
#endif
		
		case 0: {									// Software 2x zoom
			unsigned char* pd = Surf;
			unsigned char* ps = GameMem;
			if (nBurnBpp == 2) {					// 16-bit
				for (int y = 0; y < nGameHeight; y++, pd += (nPitch << nEnlarge), ps += nGameMemPitch) {
					unsigned short* psEnd = (unsigned short*)(ps + nBlitFXMemPitch);
					unsigned short* pdpc = (unsigned short*)pd;
					unsigned short* pdpn = (unsigned short*)(pd + nPitch);
					unsigned short* psp = (unsigned short*)ps;
					do {
						*pdpc++ = *psp;
						*pdpc++ = *psp;
						*pdpn++ = *psp;
						*pdpn++ = *psp++;
					} while (psp < psEnd);
				}
			} else {								// 32-bit
				if (nBurnBpp == 4) {
					for (int y = 0; y < nGameHeight; y++, pd += (nPitch << nEnlarge), ps += nGameMemPitch) {
						unsigned int* psEnd = (unsigned int*)(ps + nGameMemPitch);
						unsigned int* pdpc = (unsigned int*)pd;
						unsigned int* pdpn = (unsigned int*)(pd + nPitch);
						unsigned int* psp = (unsigned int*)ps;
						do {
							*pdpc++ = *psp;
							*pdpc++ = *psp;
							*pdpn++ = *psp;
							*pdpn++ = *psp++;
						} while (psp < psEnd);
					}
				} else {							// 24-bit
					for (int y = 0; y < nGameHeight; y++, pd += (nPitch << nEnlarge), ps += nGameMemPitch) {
						unsigned char* psEnd = (unsigned char*)(ps + nGameMemPitch);
						unsigned char* pdpc = (unsigned char*)pd;
						unsigned char* pdpn = (unsigned char*)(pd + nPitch);
						unsigned char* psp = (unsigned char*)ps;
						do {
							pdpc[0] = psp[0];
							pdpc[3] = psp[0];
							pdpn[0] = psp[0];
							pdpn[3] = psp[0];
							pdpc[1] = psp[1];
							pdpc[4] = psp[1];
							pdpn[1] = psp[1];
							pdpn[4] = psp[1];
							pdpc[2] = psp[2];
							pdpc[5] = psp[2];
							pdpn[2] = psp[2];
							pdpn[5] = psp[2];
							psp += 3;
							pdpc += 3;
							pdpn += 3;
						} while (psp < psEnd);
					}
				}
			}			
			break;
		}

#ifdef __GNUC__
		case 1: {									// AdvanceMAME 2xScale blitter (16/32BPP only)
			unsigned char* pd = Surf;
			unsigned char* psp = GameMem;
			unsigned char* psc = GameMem;
			unsigned char* psn = GameMem + nGameMemPitch;
			if (nBurnBpp == 2) {
				internal_scale2x_16_mmx((unsigned short*)pd, (unsigned short*)(pd + nPitch), (unsigned short*)psp, (unsigned short*)psc, (unsigned short*)psn, nGameMemPitch >> 1);
			} else {
				internal_scale2x_32_mmx((long unsigned int*)pd, (long unsigned int*)(pd + nPitch), (long unsigned int*)psp, (long unsigned int*)psc, (long unsigned int*)psn, nGameMemPitch >> 1);
			}
			psp -= nGameMemPitch;

			if (nBurnBpp == 2) {
				for (int y = 1; y < (nGameHeight - 1); y++) {
					pd += (nPitch << 1);
					psp += nGameMemPitch;
					psc += nGameMemPitch;
					psn += nGameMemPitch;
					internal_scale2x_16_mmx((unsigned short*)pd, (unsigned short*)(pd + nPitch), (unsigned short*)psp, (unsigned short*)psc, (unsigned short*)psn, nGameMemPitch >> 1);
				}
			} else {
				for (int y = 1; y < (nGameHeight - 1); y++) {
					pd += (nPitch << 1);
					psp += nGameMemPitch;
					psc += nGameMemPitch;
					psn += nGameMemPitch;
					internal_scale2x_32_mmx((long unsigned int*)pd, (long unsigned int*)(pd + nPitch), (long unsigned int*)psp, (long unsigned int*)psc, (long unsigned int*)psn, nGameMemPitch >> 1);
				}
			}

			pd += (nPitch << 1);
			psp += nGameMemPitch;
			psc += nGameMemPitch;
			if (nBurnBpp == 2) {
				internal_scale2x_16_mmx((unsigned short*)pd, (unsigned short*)(pd + nPitch), (unsigned short*)psp, (unsigned short*)psc, (unsigned short*)psn, nGameMemPitch >> 1);
			} else {
				internal_scale2x_32_mmx((long unsigned int*)pd, (long unsigned int*)(pd + nPitch), (long unsigned int*)psp, (long unsigned int*)psc, (long unsigned int*)psn, nGameMemPitch >> 1);
			}
			break;
		}
#endif

		case 2: {									// Super Eagle blitter (15/16BPP only)
			unsigned char* pd = Surf + 2;
			unsigned char* ps = GameMem;
			unsigned char* px = BlitFXXBuffer;
			for (int y = 0; y < nGameHeight; y++, pd += (nPitch << 1), ps += nGameMemPitch, px += nGameMemPitch) {
				_2xSaISuperEagleLine(ps, px, nGameMemPitch, (nGameMemPitch >> 1) - 1, pd, nPitch, 0);
			}
			break;
		}
		case 3: {									// 2xSaI blitter (15/16BPP only)
			unsigned char* pd = Surf + 2;
			unsigned char* ps = GameMem;
			unsigned char* px = BlitFXXBuffer;
			for (int y = 0; y < nGameHeight; y++, pd += (nPitch << 1), ps += nGameMemPitch, px += nGameMemPitch) {
				_2xSaILine(ps, px, nGameMemPitch, (nGameMemPitch >> 1) - 1, pd, nPitch, 0);
			}
			break;
		}
		case 4: {									// Super 2xSaI blitter (15/16BPP only)
			unsigned char* pd = Surf + 2;
			unsigned char* ps = GameMem;
			unsigned char* px = BlitFXXBuffer;
			for (int y = 0; y < nGameHeight; y++, pd += (nPitch << 1), ps += nGameMemPitch, px += nGameMemPitch) {
				_2xSaISuper2xSaILine(ps, px, nGameMemPitch, (nGameMemPitch >> 1) - 1, pd, nPitch, 0);
			}
			break;
		}
	}

	if (nUseSys == 0 && nDirectAccess == 0) {
		DDSURFACEDESC2 ddsdVid;
		
		// Lock the surface so we can write to it
		memset(&ddsdVid, 0, sizeof(ddsdVid));
		ddsdVid.dwSize = sizeof(ddsdVid);
		pddsBlitFX[0]->Lock(NULL, &ddsdVid, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL);
		if (ddsdVid.lpSurface == NULL) {
			return 1;
		}

		unsigned char *pd, *ps;

		unsigned char* VidSurf = (unsigned char*)ddsdVid.lpSurface;
		int nVidPitch = ddsdVid.lPitch;
		int nHeight = nGameHeight << nEnlarge;

		pd = VidSurf; ps = Surf;
		for (int y = 0; y < nHeight; y++, pd += nVidPitch, ps += nPitch) {
			memcpy(pd, ps, nPitch);
		}
		
		pddsBlitFX[0]->Unlock(NULL);
	}

	pddsBlitFX[1 ^ nDirectAccess]->Unlock(NULL);

	return 0;
}

// Run one frame and render the screen
int Frame(bool bRedraw)								// bRedraw = 0
{
	if (BlitFXMem == NULL) {
		return 1;
	}

	if (BlitFXPrim->IsLost()) {						// We've lost control of the screen
		
		if (VidSRestoreOSD()) {
			return 1;
		}
		
		if (FAILED(BlitFXDD->RestoreAllSurfaces())) {
			return 1;
		}
		
		PrimClear();
	}

	pSShotBuffer = NULL;
	pBurnDraw = BlitFXMem;
	nBurnPitch = nBlitFXMemPitch;

	if (bDrvOkay) {
		if (bRedraw) {								// Redraw current frame
			if (BurnDrvRedraw()) {
				BurnDrvFrame();						// No redraw function provided, advance one frame
			}
		} else {
			BurnDrvFrame();							// Run one frame and draw the screen
		}
	} else {
		// Blank the screen
		memset(BlitFXMem, 0, nBlitFXMemPitch * nBlitFXHeight);
	}
	pSShotBuffer = pBurnDraw;
	pBurnDraw = NULL;
	nBurnPitch = 0;

	MemToSurf();									// Copy the memory buffer to the directdraw buffer for later blitting

	return 0;
}

// Paint the BlitFX surface onto the primary surface
static int Paint(int bValidate)
{
	RECT Dest = {0, 0, 0, 0};

	if (BlitFXPrim == NULL || pddsBlitFX[nUseSys] == NULL) {
		return 1;
	}

	if (BlitFXPrim->IsLost()) {						// We've lost control of the screen
		return 1;
	}

	GetClientScreenRect(hVidWnd, &Dest);
	if (!nVidFullscreen) {
		Dest.top += nMenuHeight;
	}

	if ((nVidBlitterOpt[nVidSelect] & 0x0100)) {	// Autosize
		Dest.left += (Dest.right - Dest.left) / 2;
		Dest.left -= nGameWidth;
		Dest.right = Dest.left + (nGameWidth << 1);

		Dest.top += (Dest.bottom - Dest.top) / 2;
		Dest.top -= nGameHeight;
		Dest.bottom = Dest.top + (nGameHeight << 1);
	} else {
		if (bVidArcaderes && nVidFullscreen) {
			Dest.left = (Dest.right + Dest.left) / 2;
			Dest.left -= nGameWidth / 2;
			Dest.right = Dest.left + nGameWidth;

			Dest.top = (Dest.top + Dest.bottom) / 2;
			Dest.top -= nGameHeight / 2;
			Dest.bottom = Dest.top + nGameHeight;
		} else {
			if (!bVidFullStretch) {
				VidSScaleImage(&Dest, nGameWidth, nGameHeight, false);
			}
		}
	}

	if (bValidate & 2) {
		MemToSurf();									// Copy the memory buffer to the directdraw buffer for later blitting
	}

	RECT rect = { 0, 0, nGameWidth, nGameHeight };
	if (nEnlarge) {
		rect.right <<= 1;
		rect.bottom <<= 1;
	}

	// Display OSD text message
	VidSDisplayOSD(pddsBlitFX[nUseSys], &rect);
	
	if (BlitFXBack != NULL) {
		// Triple bufferring
		if (BlitFXBack->Blt(&Dest, pddsBlitFX[nUseSys], NULL, DDBLT_WAIT, NULL) < 0) {
			return 1;
		}
		BlitFXPrim->Flip(NULL, DDFLIP_WAIT);
	} else {
		// Normal
		if (BlitFXPrim->Blt(&Dest, pddsBlitFX[nUseSys], NULL, DDBLT_WAIT, NULL) < 0) {
			return 1;
		}
	}

	if (bValidate & 1) {
		// Validate the rectangle we just drew
		POINT c = {0, 0};
		ClientToScreen(hVidWnd, &c);
		Dest.left -= c.x; Dest.right -= c.x;
		Dest.top -= c.y; Dest.bottom -= c.y;
		ValidateRect(hVidWnd, &Dest);
	}

	return 0;
}

static int Scale(RECT* pRect, int nGameBMWidth, int nGameBMHeight)
{
	if ((nVidBlitterOpt[nVidSelect] & 0x0100) == 0) {
		return VidSScaleImage(pRect, nGameBMWidth, nGameBMHeight, false);
	}
	
	int nWidth, nHeight;
	nWidth = pRect->right - pRect->left;
	nHeight = pRect->bottom - pRect->top;

	if ((nWidth >= (nGameBMWidth << 1)) && (nHeight >= (nGameBMHeight << 1))) {
		nWidth = nGameBMWidth << 1;
		nHeight = nGameBMHeight << 1;
		nEnlarge = 1;
	} else {
		nWidth = nGameBMWidth;
		nHeight = nGameBMHeight;
		nEnlarge = 0;
	}

	pRect->left = (pRect->right - pRect->left) / 2;
	pRect->left -= nWidth / 2;
	pRect->right = pRect->left + nWidth;

	pRect->top = (pRect->bottom - pRect->top) / 2;
	pRect->top -= nHeight / 2;
	pRect->bottom = pRect->top + nHeight;

	return 0;

}

// The Video Output plugin:
struct VidOut VidOutBlitFX = {Init, Exit, Frame, Paint, Scale};

